<?php
class VIA_Ebay_Block_Adminhtml_Product_Grid extends Mage_Adminhtml_Block_Widget_Grid {
	/**
	 *
	 * @var VIA_Ebay_Helper_Data $_helper
	 */
	protected $_helper;
	
	/**
	 * Constructor.
	 */
	public function __construct() {
		parent::__construct ();
		$this->setId ( 'viaebay_product_grid' );
		$this->setDefaultSort ( 'entity_id' );
		$this->setDefaultDir ( 'ASC' );
		$this->setUseAjax ( true );
		$this->setSaveParametersInSession ( true );
		
		$this->_helper = $this->helper ( 'viaebay/data' );
	}
	
	/**
	 *
	 * {@inheritDoc}
	 *
	 * @see Mage_Adminhtml_Block_Widget_Grid::_prepareCollection()
	 */
	protected function _prepareCollection() {
		$store = $this->_helper->getVIAStore ();
		$viaStoreId = $this->_helper->getVIAStoreId ();
		/* @var $collection Mage_Catalog_Model_Resource_Product_Collection */
		$collection = Mage::getModel ( 'viaebay/product' )->getCollection ();
		$collection->setStoreId ( $viaStoreId );
		// $collection->addAttributeToSelect ( 'sku' )->addAttributeToSelect ( 'name' )->addAttributeToSelect ( 'attribute_set_id' )->addAttributeToSelect ( 'type_id' )->addAttributeToSelect ( 'viaebay_export' )->addAttributeToSelect ( 'viaebay_id' );
		$collection->addAttributeToSelect ( '*' );
		
		if (Mage::helper ( 'catalog' )->isModuleEnabled ( 'Mage_CatalogInventory' )) {
			$collection->joinField ( 'qty', 'cataloginventory/stock_item', 'qty', 'product_id=entity_id', '{{table}}.stock_id=1', 'left' );
		}
		if ($store->getId ()) {
			// $collection->setStoreId($store->getId());
			$adminStore = Mage_Core_Model_App::ADMIN_STORE_ID;
			$collection->addStoreFilter ( $viaStoreId );
			$collection->joinAttribute ( 'name', 'catalog_product/name', 'entity_id', null, 'inner', $viaStoreId );
			$collection->joinAttribute ( 'status', 'catalog_product/status', 'entity_id', null, 'inner', $viaStoreId );
			$collection->joinAttribute ( 'visibility', 'catalog_product/visibility', 'entity_id', null, 'inner', $viaStoreId );
			$collection->joinAttribute ( 'price', 'catalog_product/price', 'entity_id', null, 'left', $viaStoreId );
		} else {
			$collection->addAttributeToSelect ( 'price' );
			$collection->joinAttribute ( 'status', 'catalog_product/status', 'entity_id', null, 'inner' );
			$collection->joinAttribute ( 'visibility', 'catalog_product/visibility', 'entity_id', null, 'inner' );
		}
		$this->setCollection ( $collection );
		
		parent::_prepareCollection ();
		$this->getCollection ()->addWebsiteNamesToResult ();
		return $this;
	}
	
	/**
	 * Prepare columns.
	 *
	 * {@inheritDoc}
	 *
	 * @see Mage_Adminhtml_Block_Widget_Grid::_prepareColumns()
	 */
	protected function _prepareColumns() {
		$this->addColumn ( 'entity_id', array (
				'header' => Mage::helper ( 'viaebay' )->__ ( 'ID' ),
				'type' => 'number',
				'index' => 'entity_id' 
		) );
		
		$this->addColumn ( 'name', array (
				'header' => Mage::helper ( 'viaebay' )->__ ( 'Name' ),
				'index' => 'name' 
		) );
		
		$this->addColumn ( 'type', array (
				'header' => Mage::helper ( 'catalog' )->__ ( 'Type' ),
				'width' => '60px',
				'index' => 'type_id',
				'type' => 'options',
				'options' => Mage::getSingleton ( 'catalog/product_type' )->getOptionArray () 
		) );
		
		$sets = Mage::getResourceModel ( 'eav/entity_attribute_set_collection' )->setEntityTypeFilter ( Mage::getModel ( 'catalog/product' )->getResource ()->getTypeId () )->load ()->toOptionHash ();
		
		$this->addColumn ( 'set_name', array (
				'header' => Mage::helper ( 'catalog' )->__ ( 'Attrib. Set Name' ),
				'width' => '100px',
				'index' => 'attribute_set_id',
				'type' => 'options',
				'options' => $sets 
		) );
		
		$this->addColumn ( 'sku', array (
				'header' => Mage::helper ( 'viaebay' )->__ ( 'SKU' ),
				'index' => 'sku' 
		) );
		
		$store = $this->_getStore ();
		$this->addColumn ( 'price', array (
				'header' => Mage::helper ( 'viaebay' )->__ ( 'Price' ),
				'type' => 'price',
				'currency_code' => $store->getBaseCurrency ()->getCode (),
				'index' => 'price' 
		) );
		
		if (Mage::helper ( 'catalog' )->isModuleEnabled ( 'Mage_CatalogInventory' )) {
			$this->addColumn ( 'qty', array (
					'header' => Mage::helper ( 'viaebay' )->__ ( 'Stock Level' ),
					'width' => '20',
					'type' => 'number',
					'index' => 'qty' 
			) );
		}
		
		$this->addColumn ( 'visibility', array (
				'header' => Mage::helper ( 'catalog' )->__ ( 'Visibility' ),
				'width' => '70px',
				'index' => 'visibility',
				'type' => 'options',
				'options' => Mage::getModel ( 'catalog/product_visibility' )->getOptionArray () 
		) );
		
		$this->addColumn ( 'status', array (
				'header' => Mage::helper ( 'catalog' )->__ ( 'Status' ),
				'width' => '70px',
				'index' => 'status',
				'type' => 'options',
				'options' => Mage::getSingleton ( 'catalog/product_status' )->getOptionArray () 
		) );
		
		$this->addColumn ( 'viaebay_export', array (
				'header' => Mage::helper ( 'viaebay' )->__ ( 'Exported to VIA-Connect' ),
				'width' => '70px',
				'index' => 'viaebay_export',
				'type' => 'options',
				'options' => array (
						0 => 'No',
						1 => 'Yes' 
				) 
		) );
		
		$this->addColumn ( 'viaebay_id', array (
				'header' => Mage::helper ( 'viaebay' )->__ ( 'VIA-Connect #' ),
				'index' => 'viaebay_id',
				'type' => 'number' 
		) );
		
		return parent::_prepareColumns ();
	}
	
	/**
	 * Prepare Massaction.
	 */
	protected function _prepareMassaction() {
		$yesnoSource = Mage::getModel ( 'adminhtml/system_config_source_yesno' )->toOptionArray ();
		
		$this->setMassactionIdField ( 'entity_id' );
		$this->getMassactionBlock ()->setFormFieldName ( 'productIds' );
		
		$this->getMassactionBlock ()->addItem ( 'enable', array (
				'label' => Mage::helper ( 'viaebay' )->__ ( 'Transfer Products' ),
				'url' => $this->getUrl ( 'adminhtml/viaebay_product/massEnable', array () ),
				'confirm' => Mage::helper ( 'viaebay' )->__ ( 'Are you sure? Direct transfers are only recommended for small sets of products. For large sets use the Async. method instead.' ),
				'additional' => array (
						'async' => array (
								'name' => 'async',
								'type' => 'select',
								'class' => 'required-entry',
								'label' => Mage::helper ( 'viaebay' )->__ ( 'Async' ),
								'values' => $yesnoSource 
						) 
				) 
		) );
		
		$this->getMassactionBlock ()->addItem ( 'transfer', array (
				'label' => Mage::helper ( 'viaebay' )->__ ( 'Update Products' ),
				'url' => $this->getUrl ( 'adminhtml/viaebay_product/massTransfer', array () ),
				'confirm' => Mage::helper ( 'viaebay' )->__ ( 'Are you sure? Direct updates are only recommended for small sets of products. For large sets use the Async. method instead.' ),
				'additional' => array (
						'async' => array (
								'name' => 'async',
								'type' => 'select',
								'class' => 'required-entry',
								'label' => Mage::helper ( 'viaebay' )->__ ( 'Async' ),
								'values' => $yesnoSource 
						) 
				) 
		) );
		
		$this->getMassactionBlock ()->addItem ( 'disableDivider', array (
				'label' => '--------',
				'url' => $this->getUrl ( '*/*/index', array (
						'_current' => true 
				) ),
				'callback' => "null" 
		) );
		
		$this->getMassactionBlock ()->addItem ( 'disable', array (
				'label' => Mage::helper ( 'viaebay' )->__ ( 'End Products' ),
				'url' => $this->getUrl ( 'adminhtml/viaebay_product/massDisable', array () ),
				'confirm' => Mage::helper ( 'viaebay' )->__ ( 'Are you sure?' ),
				'additional' => array (
						'async' => array (
								'name' => 'async',
								'type' => 'select',
								'class' => 'required-entry',
								'label' => Mage::helper ( 'viaebay' )->__ ( 'Async' ),
								'values' => $yesnoSource 
						) 
				) 
		) );
		
		return $this;
	}
	
	/**
	 * Get customized row url.
	 *
	 * {@inheritDoc}
	 *
	 * @see Mage_Adminhtml_Block_Widget_Grid::getRowUrl()
	 */
	public function getRowUrl($row) {
		return 'javascript:viaebayOpenGridUrl(\'' . $this->getUrl ( 'adminhtml/catalog_product/edit', array (
				'id' => $row->getEntityId (),
				'popup' => true 
		) ) . '\')';
	}
	
	/**
	 * Get grid update url.
	 *
	 * {@inheritDoc}
	 *
	 * @see Mage_Adminhtml_Block_Widget_Grid::getGridUrl()
	 */
	public function getGridUrl() {
		return $this->getUrl ( 'adminhtml/viaebay_product/grid', array (
				'_current' => true 
		) );
	}
	
	/**
	 * Get current store.
	 */
	protected function _getStore() {
		$storeId = ( int ) $this->getRequest ()->getParam ( 'store', 0 );
		return Mage::app ()->getStore ( $storeId );
	}
}