<?php
class VIA_Ebay_Block_Adminhtml_Sales_Order_Grid extends Mage_Adminhtml_Block_Widget_Grid {
	/**
	 *
	 * @var VIA_Ebay_Helper_Data $_helper
	 */
	protected $_helper;
	
	/**
	 * Constructor.
	 */
	public function __construct() {
		parent::__construct ();
		$this->setId ( 'viaebay_sales_order_grid' );
		$this->setDefaultSort ( 'checkout_complete_date' );
		$this->setDefaultDir ( 'DESC' );
		$this->setUseAjax ( true );
		$this->setSaveParametersInSession ( true );
		
		$this->_helper = $this->helper ( 'viaebay/data' );
	}
	
	/**
	 *
	 * {@inheritdoc}
	 *
	 * @see Mage_Adminhtml_Block_Widget_Grid::_prepareCollection()
	 */
	protected function _prepareCollection() {
		$magentoOrderTable = Mage::getSingleton ( 'core/resource' )->getTableName ( 'sales/order' );
		
		/* @var $collection VIA_Ebay_Model_Resource_Order_Collection */
		$collection = Mage::getModel ( 'viaebay/order' )->getCollection ();
		
		$collection->getSelect ()->joinLeft ( array (
				'magento_order_table' => $magentoOrderTable 
		), 'main_table.magento_order_id=magento_order_table.entity_id', array (
				'magento_order_number' => 'magento_order_table.increment_id' 
		) );
		
		$this->setCollection ( $collection );
		return parent::_prepareCollection ();
	}
	
	/**
	 * Prepare columns.
	 *
	 * {@inheritdoc}
	 *
	 * @see Mage_Adminhtml_Block_Widget_Grid::_prepareColumns()
	 */
	protected function _prepareColumns() {
		$this->addColumn ( 'order_id', array (
				'header' => $this->_helper->__ ( 'ID' ),
				'type' => 'number',
				'index' => 'order_id' 
		) );
		
		$this->addColumn ( 'checkout_complete_date', array (
				'header' => $this->_helper->__ ( 'Checkout Complete Date' ),
				'index' => 'checkout_complete_date',
				'type' => 'datetime' 
		) );
		
		$this->addColumn ( 'viaebay_order_id', array (
				'header' => $this->_helper->__ ( 'VIA-Connect Order Id' ),
				'index' => 'viaebay_order_id',
				'type' => 'number' 
		) );
		
		/*
		 * $this->addColumn ( 'magento_order_id', array (
		 * 'header' => $this->_helper->__ ( 'Magento Order Id' ),
		 * 'renderer' => 'viaebay/adminhtml_widget_grid_column_renderer_magentoOrderId',
		 * 'index' => 'magento_order_id',
		 * 'filter_index' => 'magento_order_table.entity_id',
		 * 'type' => 'number'
		 * ) );
		 */
		
		$this->addColumn ( 'magento_order_number', array (
				'header' => $this->_helper->__ ( 'Magento Order Number' ),
				'renderer' => 'viaebay/adminhtml_widget_grid_column_renderer_magentoOrderNumber',
				'index' => 'magento_order_number',
				'filter_index' => 'magento_order_table.increment_id' 
		) );
		
		$this->addColumn ( 'platform_order_id', array (
				'header' => $this->_helper->__ ( 'Platform Order Id' ),
				'renderer' => 'viaebay/adminhtml_widget_grid_column_renderer_platformOrderId',
				'index' => 'platform_order_id' 
		) );
		
		$this->addColumn ( 'buyer_name', array (
				'header' => $this->_helper->__ ( 'Buyer Name' ),
				'index' => 'buyer_name' 
		) );
		
		$this->addColumn ( 'import_status', array (
				'header' => $this->_helper->__ ( 'Import Status' ),
				'width' => '100',
				'renderer' => 'viaebay/adminhtml_widget_grid_column_renderer_status',
				'filter' => false,
				'sortable' => false,
				'index' => 'magento_order_id' 
		) );
		
		$this->addColumn ( 'action', array (
				'header' => $this->_helper->__ ( 'Action' ),
				'width' => '100',
				'renderer' => 'viaebay/adminhtml_widget_grid_column_renderer_reimport',
				'filter' => false,
				'sortable' => false,
				'index' => 'viaebay_order_id' 
		) );
		
		return parent::_prepareColumns ();
	}
	
	/**
	 * Prepare Massaction.
	 */
	protected function _prepareMassaction() {
		$this->setMassactionIdField ( 'viaebay_order_id' );
		$this->getMassactionBlock ()->setFormFieldName ( 'viaebay_order_id' );
		
		return $this;
	}
	
	/**
	 * Used for AJAX loading.
	 */
	public function getGridUrl() {
		return $this->getUrl ( '*/*/grid', array (
				'_current' => true 
		) );
	}
}