<?php
class VIA_Ebay_Helper_Cron extends Mage_Core_Helper_Abstract {
	/**
	 * Schedule jobs for next cron run.
	 *
	 * @param array $cronjobs
	 *        	to schedule
	 * @param string $addMessage
	 *        	add messages to session
	 */
	public function scheduleJobs(array $cronjobs, $addMessage = true) {
		$session = Mage::getSingleton ( 'adminhtml/session' );
		/* @var $session Mage_Adminhtml_Model_Session */
		
		$time = time ();
		$timeString = strftime ( '%Y-%m-%d %H:%M:%S', $time );
		foreach ( $cronjobs as $code ) {
			$schedule = Mage::getModel ( 'cron/schedule' );
			$schedule->setJobCode ( $code );
			$schedule->setStatus ( Mage_Cron_Model_Schedule::STATUS_PENDING );
			$schedule->setCreatedAt ( $timeString );
			$schedule->setScheduledAt ( $timeString );
			$schedule->save ();
			if ($addMessage) {
				$session->addSuccess ( $this->__ ( 'Scheduled "%s" at %s', $code, $timeString ) );
			}
		}
	}
	/**
	 * Run jobs now.
	 *
	 * @param array $cronjobs
	 *        	to run
	 * @param string $addMessage
	 *        	add messages to session
	 */
	public function runJobs(array $cronjobs, $addMessage = true) {
		$session = Mage::getSingleton ( 'adminhtml/session' );
		/* @var $session Mage_Adminhtml_Model_Session */
		
		foreach ( $cronjobs as $code ) {
			try {
				switch ($code) {
					case VIA_Ebay_Model_Cron::SYNC_CATALOG :
						$res = Mage::getSingleton ( 'viaebay/cron' )->syncCatalog ();
						if (is_object ( $res )) {
							$session->addSuccess ( $this->__ ( 'Products added (%s) updated (%s)  deleted (%s)', $res->product->added, $res->product->updated, $res->product->deleted ) );
							$session->addSuccess ( $this->__ ( 'Categories added (%s) deleted (%s)', $res->category->added, $res->category->deleted ) );
							if ($res->product->error > 0) {
								$session->addError ( $this->__ ( 'Errors (%s)', $res->product->error ) );
							}
						}
						
						break;
					case VIA_Ebay_Model_Cron::SYNC_ORDERS :
						$res = Mage::getSingleton ( 'viaebay/cron' )->syncOrders ();
						if (is_object ( $res )) {
							$session->addSuccess ( $this->__ ( 'Imported orders (%s)', $res->added ) );
							$session->addSuccess ( $this->__ ( 'Paid orders (%s) ', $res->paid ) );
							if ($res->error > 0) {
								$session->addError ( $this->__ ( 'Errors (%s)', $res->error ) );
							}
						}
						break;
				}
			} catch ( Exception $e ) {
				$session->addError ( $e->getMessage () );
				Mage::helper ( 'viaebay/log' )->logException ( $e );
			}
		}
	}
}
