<?php
class VIA_Ebay_Helper_Log extends Mage_Core_Helper_Abstract {
	const LOGGING_ENABLED = 'viaebay/extended/enable_logging';
	const MAX_LOG_LEVEL = 'viaebay/extended/max_log_level';
	
	/**
	 * Logfile used.
	 *
	 * @var string
	 */
	const LOGFILE = 'viaebay.log';
	
	/**
	 *
	 * @return boolean
	 */
	public function isLoggingEnabled() {
		return Mage::getStoreConfigFlag ( self::LOGGING_ENABLED );
	}
	
	/**
	 * Log exceptions
	 *
	 * @param Exception $e        	
	 * @param bool $logToSession        	
	 */
	public function logException(Exception $e, $logToSession = true) {
		if ($logToSession) {
			$this->logToSession ( $e->__toString (), Zend_Log::ERR );
		}
		$this->logToFile ( "\n" . $e->__toString (), Zend_Log::ERR );
	}
	/**
	 * Log to Mage::log
	 *
	 * @param string $message        	
	 * @param string $level        	
	 */
	public function log($message, $level = null) {
		$this->logToSession ( $message, $level );
		$this->logToFile ( $message, $level );
	}
	
	/**
	 * Log to file only.
	 *
	 * @param string $message        	
	 * @param string $level        	
	 */
	public function logToFile($message, $level = null) {
		$level = is_null ( $level ) ? Zend_Log::DEBUG : ( int ) $level;
		
		if (! $this->isLoggingEnabled ()) {
			return;
		}
		
		$maxLogLevel = ( int ) Mage::getStoreConfig ( self::MAX_LOG_LEVEL );
		
		if ($maxLogLevel >= $level) {
			Mage::log ( $message, $level, self::LOGFILE, true );
		}
	}
	
	/**
	 * Log to file only.
	 *
	 * @param array $strackTrace        	
	 * @param string $level        	
	 */
	public function logStackTrace(array $strackTrace, $level = null) {
		$level = is_null ( $level ) ? Zend_Log::DEBUG : ( int ) $level;
		
		if (! $this->isLoggingEnabled ()) {
			return;
		}
		
		$maxLogLevel = ( int ) Mage::getStoreConfig ( self::MAX_LOG_LEVEL );
		
		if ($maxLogLevel >= $level) {
			Mage::log ( print_r ( $strackTrace, true ), $level, self::LOGFILE, true );
		}
	}
	
	/**
	 * Log to session only.
	 *
	 * @param string $message        	
	 * @param string $level        	
	 */
	public function logToSession($message, $level = null) {
		$level = is_null ( $level ) ? Zend_Log::DEBUG : ( int ) $level;
		
		$adminSession = Mage::getSingleton ( 'admin/session' );
		/* @var $adminSession Mage_Admin_Model_Session */
		
		// verify if the user is logged in to the backend
		if ($adminSession->isLoggedIn ()) {
			$adminhtmlSession = Mage::getSingleton ( 'adminhtml/session' );
			
			switch ($level) {
				case Zend_Log::EMERG :
					$adminhtmlSession->addError ( $message );
					break;
				case Zend_Log::ALERT :
					$adminhtmlSession->addError ( $message );
					break;
				case Zend_Log::CRIT :
					$adminhtmlSession->addError ( $message );
					break;
				case Zend_Log::ERR :
					$adminhtmlSession->addError ( $message );
					break;
				case Zend_Log::WARN :
					$adminhtmlSession->addWarning ( $message );
					break;
			}
		}
	}
}
