<?php
class VIA_Ebay_Helper_Product extends Mage_Core_Helper_Abstract {
	/**
	 * Extract mapped attributes from Product.
	 *
	 * @param Mage_Catalog_Model_Product $product
	 *        	to extract mapped values from.
	 * @return array of name => value
	 */
	public function getMappedAttributeValues(Mage_Catalog_Model_Product $product, Mage_Catalog_Model_Product $parentProduct = null) {
		$useConfigurablePrice = Mage::getStoreConfigFlag ( 'viaebay/catalog/use_configurable_product_price' );

		$mapping = Mage::getModel ( 'viaebay/system_config_source_attributes' )->toSelectArray ();

		$result = array ();

		// Fill empty mappings
		foreach ( $mapping as $key => $value ) {
			$result [$key] = null;
		}


		$result ['_id'] = $product->getId ();
		if ($parentProduct != null) {
			$result ['_parent_id'] = $parentProduct->getId ();
		}

		// Default price
		$result ['Price'] = Mage::helper ( 'tax' )->getPrice ( $product, $product->getPrice (), true );

		foreach ( $product->getAttributes () as $attribute ) {
			/* @var $attribute Mage_Eav_Model_Attribute */
			if (strlen ( $attribute->getViaebayAttribute () ) <= 0 || $attribute->getViaebayAttribute () == 'None') {
				continue;
			}

			//var_dump ( $attribute->getViaebayAttribute () );

			if ($attribute->getViaebayAttribute () == 'Price') {
				if ($useConfigurablePrice && $parentProduct != null) {
					$result [$attribute->getViaebayAttribute ()] = Mage::helper ( 'tax' )->getPrice ( $parentProduct, $attribute->getFrontend ()->getValue ( $parentProduct ), true );
				} else {
					$result [$attribute->getViaebayAttribute ()] = Mage::helper ( 'tax' )->getPrice ( $product, $attribute->getFrontend ()->getValue ( $product ), true );
				}
			} else if ($attribute->getViaebayAttribute () == 'DiscountOffer') {
				if ($useConfigurablePrice && $parentProduct != null) {
					$result [$attribute->getViaebayAttribute ()] = Mage::helper ( 'tax' )->getPrice ( $parentProduct, $parentProduct->getData ( $attribute->getAttributeCode () ), true );
				} else {
					$result [$attribute->getViaebayAttribute ()] = Mage::helper ( 'tax' )->getPrice ( $product, $product->getData ( $attribute->getAttributeCode () ), true );
				}
			} else if ($attribute->getViaebayAttribute () != 'CustomAttribute') {
				$result [$attribute->getViaebayAttribute ()] = $attribute->getFrontend ()->getValue ( $product );
			}
		}

		// if ($parentProduct!= null) {
		// var_dump($result);
		// }

		return $result;
	}
}
