<?php
/**
 * 
 */
class VIA_Ebay_Model_Carrier extends Mage_Shipping_Model_Carrier_Abstract implements Mage_Shipping_Model_Carrier_Interface {
	protected $_code = 'viaebay';
	protected $_isFixed = true;
	protected static $METHODS = array (
			'DE_DeutschePostBrief' => 'Deutsche Post Brief',
			'DE_DHLPackchen' => 'DHL Päckchen',
			'DE_Paket' => 'Paketversand',
			'DE_Express' => 'Expressversand',
			'DE_DHLPaket' => 'DHL Paket',
			'DE_DPBuecherWarensendung' => 'Deutsche Post Bücher-/Warensendung',
			'DE_HermesPaket' => 'Hermes Paket',
			'DE_IloxxTransport' => 'iloxx Transport',
			'DE_SonstigeDomestic' => 'Sonstige 1-2 Werktage',
			'DE_Einschreiben' => 'Einschreiben (inkl. aller Gebühren)',
			'DE_Nachname' => 'Nachnahme (inkl. aller Gebühren)',
			'DE_SpecialDelivery' => 'Sonderversand',
			'DE_SparversandAusDemAusland' => 'Sparversand aus dem Ausland',
			'DE_StandardversandAusDemAusland' => 'Standardversand aus dem Ausland',
			'DE_ExpressversandAusDemAusland' => 'Expressversand aus dem Ausland',
			'DE_DPDExpress' => 'DPD Express',
			'DE_UPSExpress' => 'UPS Express',
			'DE_GLSPaket' => 'GLS Paket',
			'DE_EbayHermesPaketShop2Shop' => 'eBay Hermes Paket Shop2Shop',
			'DE_eBayHermesPaketSperrgutShop2Shop' => 'eBay Hermes Paket Sperrgut Shop2Shop (Abgabe und Zustellung im Paketshop)',
			'DE_HermesPaketSperrgut' => 'Hermes Paket Sperrgut',
			'DE_DPDClassic' => 'DPD Classic',
			'DE_UPSStandard' => 'UPS Standard',
			'DE_DeutschePostBriefInternational' => 'Deutsche Post Brief',
			'DE_DHLPackchenInternational' => 'DHL Päckchen International',
			'DE_PaketInternational' => 'Standard Int\'l Versand',
			'DE_ExpressInternational' => 'Express Int\'l Versand',
			'DE_DHLPaketInternational' => 'DHL Paket International',
			'DE_IloxxTransportInternational' => 'iloxx Transport International',
			'DE_SonstigeInternational' => 'Economy Int\'l Versand',
			'DE_HermesPaketInternational' => 'Hermes Paket International',
			'DE_DeutschePostBriefIntlEcon' => 'Deutsche Post Presse und Bücher Economy',
			'DE_DeutschePostBriefIntlPri' => 'Deutsche Post Presse und Bücher Priority',
			'PromotionalShippingMethod' => 'Versand zum Sonderpreis' 
	);
	public function collectRates(Mage_Shipping_Model_Rate_Request $request) {
		if (! $this->getConfigFlag ( 'active' ) || ! in_array ( $request->getStoreId (), array (
				Mage_Core_Model_App::ADMIN_STORE_ID,
				Mage::helper ( 'viaebay/data' )->getVIAStoreId () 
		) )) {
			return false;
		}
		
		/* @var $result Mage_Shipping_Model_Rate_Result */
		$result = Mage::getModel ( 'shipping/rate_result' );
		
		foreach ( $this->getAllowedMethods () as $code => $title ) {
			
			/* @var $method Mage_Shipping_Model_Rate_Result_Method */
			$method = Mage::getModel ( 'shipping/rate_result_method' );
			
			$method->setCarrier ( 'viaebay' );
			$method->setCarrierTitle ( $this->getConfigData ( 'title' ) );
			
			$method->setMethod ( $code );
			$method->setMethodTitle ( $title );
			
			$shippingPrice = $this->getConfigData ( 'price' );
			
			$method->setPrice ( max ( 0, $shippingPrice ) );
			$method->setCost ( max ( 0, $shippingPrice ) );
			
			$result->append ( $method );
		}
		
		return $result;
	}
	
	/**
	 * This method is used when viewing shipping methods.
	 */
	public function getAllowedMethods() {
		$allowedMethods = explode ( ",", $this->getConfigData ( 'allowed_methods' ) );
		
		$result = array ();
		
		foreach ( $allowedMethods as $m ) {
			$result [] = self::$METHODS [$m];
		}
		
		return $result;
	}
	public static function getAllMethods() {
		return self::$METHODS;
	}
	public function isTrackingAvailable() {
		return true;
	}
	public function getViaebayCarrier($methodCode) {
		switch ($methodCode) {
			case 'viaebay_DE_DHLPackchen' :
			case 'viaebay_DE_DHLPaket' :
			case 'viaebay_DE_DHLPackchenInternational' :
			case 'viaebay_DE_DHLPaketInternational' :
				return 'DHL';
			case 'viaebay_DE_HermesPaket' :
			case 'viaebay_DE_EbayHermesPaketShop2Shop' :
			case 'viaebay_DE_eBayHermesPaketSperrgutShop2Shop' :
			case 'viaebay_DE_HermesPaketSperrgut' :
			case 'viaebay_DE_HermesPaketInternational' :
				return 'Hermes';
			case 'viaebay_DE_IloxxTransport' :
			case 'viaebay_DE_IloxxTransportInternational' :
				return 'iLoxx';
			case 'viaebay_DE_UPSExpress' :
			case 'viaebay_DE_UPSStandard' :
				return 'UPS';
			default :
				return 'Standard Versand';
		}
	}
}
	
