<?php
class VIA_Ebay_Model_Exception_Products extends VIA_Ebay_Model_Exception_Abstract {
	/**
	 *
	 * @var array
	 */
	private $productExceptions;
	
	/**
	 * Construct the exception
	 *
	 * @param string $msg        	
	 * @param int $code        	
	 * @param Exception $previous        	
	 * @return void
	 */
	public function __construct(array $productExceptions = array(), $msg = '', $code = 0, Exception $previous = null) {
		parent::__construct ( $msg, $code, $previous );
		$this->productExceptions = $productExceptions;
	}
	
	/**
	 * Add product exceptions.
	 *
	 * @param VIA_Ebay_Model_Exception_Product $exception        	
	 */
	public function addException(VIA_Ebay_Model_Exception_Product $exception) {
		$this->productExceptions [] = $exception;
	}
	
	/**
	 * Return first product url.
	 *
	 * @return string
	 */
	public function getProductUrl() {
		return count ( $this->productExceptions ) > 0 ? $this->productExceptions [0]->getProductUrl () : '';
	}
	/**
	 * Return if exceptions are present.
	 *
	 * @return boolean
	 */
	public function hasProductExceptions() {
		return count ( $this->productExceptions ) > 0;
	}
	
	/**
	 * Get custom message
	 *
	 * @return string
	 */
	public function __toString() {
		/* @var $helper VIA_Ebay_Helper_Data */
		$helper = Mage::helper ( 'viaebay/data' );
		$result = $this->getMessage () . "\n" . $helper->__ ( 'Products:' ) . "\n";
		/* @var $productException VIA_Ebay_Model_Exception_Product */
		foreach ( $this->productExceptions as $productException ) {
			$result .= $productException->getMessage() . "\n";
		}
		return $result;
	}
}
