<?php

/**
 * Observer for all events.
 *
 * @category	VIA
 * @package		VIA_Ebay
 * @subpackage	Model
 */
class VIA_Ebay_Model_Observer {
	private $STOCK_ATTRIBUTES = array (
			'qty',
			'manage_stock',
			'is_in_stock' 
	);
	
	/**
	 * Triggered before Layout is prepared.
	 * So no parent set yet - argh.
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return Dhl_MeinPaket_Model_Observer
	 */
	function addProductGridMassaction(Varien_Event_Observer $observer) {
		$block = $observer->getEvent ()->getBlock ();
		if ($block != null && $block instanceof Mage_Adminhtml_Block_Widget_Grid_Massaction && $block->getRequest ()->getControllerName () == 'catalog_product') {
			$block->addItem ( 'massAddToVIAEbayBacklog', array (
					'label' => Mage::helper ( 'viaebay' )->__ ( 'Add to VIA-Connect backlog' ),
					'url' => Mage::app ()->getStore ()->getUrl ( 'adminhtml/viaebay_backlog_product/massAddToBacklog' ) 
			) );
		}
		
		return $this;
	}
	
	/**
	 * Triggered after deleting but before updating database
	 *
	 * @see VIA_Ebay_Model_Observer::catalogProductSaveAfter()
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function catalogProductDeleteBefore(Varien_Event_Observer $observer) {
		try {
			$product = $observer->getEvent ()->getProduct ();
			
			$catalogService = Mage::getSingleton ( 'viaebay/service_catalog' );
			$catalogService->deleteProduct ( $product );
			$catalogService->deleteProductAsVariation ( $product );
		} catch ( Exception $ex ) {
			Mage::helper ( 'viaebay/log' )->logException ( $ex );
		}
		
		return $this;
	}
	
	/**
	 * Triggered before product is saved.
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function catalogProductSaveBefore(Varien_Event_Observer $observer) {
		/**
		 *
		 * @var $product Mage_Catalog_Model_Product
		 */
		$product = $observer->getEvent ()->getProduct ();
		/* @var $product Mage_Catalog_Model_Product */
		
		$product->setData ( 'viaebay_product_save_before_run', true );
		
		if (! $product->getId ()) {
			/*
			 * Delete viaebay data for new and as such not synchronized objects. It's assumed that viaebay_export is either set by the user or productDuplicate in this class. This is for example needed for quick created variants.
			 */
			$product->setData ( 'viaebay_id', '' );
		}
		
		try {
			$changes = array ();
			$typeInstance = $product->getTypeInstance ();
			if ($typeInstance instanceof Mage_Catalog_Model_Product_Type_Configurable) {
				// Find changed variants
				$data = $typeInstance->getProduct ( $product )->getConfigurableProductsData ();
				
				if (is_array ( $data )) {
					$new = array_keys ( $data );
					
					$old = $product->getTypeInstance ()->getUsedProductIds ();
					$insert = array_diff ( $new, $old );
					$delete = array_diff ( $old, $new );
					
					if (! empty ( $insert ) || ! empty ( $delete )) {
						// Flag forced update.
						$product->setData ( 'viaebay_force_update', true );
						return true;
					}
				}
			}
			foreach ( $product->getData () as $attribute => $newValue ) {
				// Find changed products
				$oldValue = $product->getOrigData ( $attribute );
				
				if ($oldValue == null || (is_array ( $newValue ) && is_array ( $oldValue ))) {
					// Ignored
				} else if ($newValue != $oldValue) {
					$changes [] = $attribute;
				}
			}
			
			// Flag schedule in backlog.
			$product->setData ( 'viaebay_backlog_changes', array_unique ( $changes ) );
		} catch ( Exception $e ) {
			Mage::helper ( 'viaebay/log' )->logException ( $e );
		}
		return $this;
	}
	
	/**
	 * Triggered after product is saved.
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function catalogProductSaveAfter(Varien_Event_Observer $observer) {
		try {
			$product = $observer->getEvent ()->getProduct ();
			/* @var $product Mage_Catalog_Model_Product */
			
			$product->unsetData ( 'viaebay_product_save_before_run' );
			
			$forceUpdate = $product->getData ( 'viaebay_force_update' );
			$backlogChanges = $product->getData ( 'viaebay_backlog_changes' );
			
			if ($forceUpdate) {
				// This product is forced to update now!
				$catalogService = Mage::getSingleton ( 'viaebay/service_catalog' );
				$catalogService->exportProductById ( $product->getId () );
				foreach ( Mage::helper ( 'viaebay/backlog' )->getParentIds ( $product->getId () ) as $productId ) {
					$catalogService->exportProductById ( $productId );
				}
			} else if (is_array ( $backlogChanges )) {
				// Schedule the product for later
				$changes = $product->getData ( 'viaebay_backlog_changes' );
				if (count ( $changes ) && $product->getId ()) {
					Mage::helper ( 'viaebay/backlog' )->createBacklog ( $product->getId (), implode ( ',', $changes ) );
					Mage::helper ( 'viaebay/backlog' )->createParentBacklog ( $product->getId () );
				}
			}
		} catch ( Exception $ex ) {
			Mage::helper ( 'viaebay/log' )->logException ( $ex );
		}
		
		return $this;
	}
	
	/**
	 * Triggered before product massaction.
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function catalogProductAttributeUpdateBefore(Varien_Event_Observer $observer) {
		$attributesData = $observer->getEvent ()->getAttributesData ();
		$productIds = $observer->getEvent ()->getProductIds ();
		
		$changes = implode ( ',', array_keys ( $attributesData ) );
		
		foreach ( $productIds as $id ) {
			Mage::helper ( 'viaebay/backlog' )->createBacklog ( $id, $changes );
			
			foreach ( Mage::helper ( 'viaebay/backlog' )->getParentIds ( $id ) as $parentId ) {
				Mage::helper ( 'viaebay/backlog' )->createBacklog ( $parentId );
			}
		}
		return $this;
	}
	
	/**
	 * Triggered after invoice is payed.
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function salesOrderPaymentPay(Varien_Event_Observer $observer) {
		return $this; // Handled by payment module
		
		try {
			$payment = $observer->getEvent ()->getPayment ();
			
			Mage::getSingleton ( 'viaebay/service_order' )->updatePaid ( $payment );
		} catch ( Exception $e ) {
			Mage::helper ( 'viaebay/log' )->logException ( $e );
		}
		
		return $this;
	}
	
	/**
	 * Triggered after invoice is payed.
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function salesOrderPaymentRefund(Varien_Event_Observer $observer) {
		return $this; // Handled by payment module
		
		try {
			$payment = $observer->getEvent ()->getPayment ();
			
			Mage::getSingleton ( 'viaebay/service_order' )->updatePaid ( $payment->getOrder () );
		} catch ( Exception $e ) {
			Mage::helper ( 'viaebay/log' )->logException ( $e );
		}
		
		return $this;
	}
	
	/**
	 * Triggered after stock item is saved
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function catalogInventoryStockItemSaveAfter(Varien_Event_Observer $observer) {
		try {
			$stockItem = $observer->getEvent ()->getItem ();
			/* @var $stockItem Mage_CatalogInventory_Model_Stock_Item */
			
			if ($stockItem != null) {
				$changes = array ();
				foreach ( $this->STOCK_ATTRIBUTES as $attribute ) {
					if ($stockItem->getData ( $attribute ) != $stockItem->getOrigData ( $attribute )) {
						$changes [] = $attribute;
					}
				}
				
				if (! empty ( $changes )) {
					$product = $stockItem->getProduct ();
					
					if ($product != null && $product->getData ( 'viaebay_product_save_before_run' )) {
						$product->setData ( 'viaebay_force_update', true );
					}
					/* @var $catalogService VIA_Ebay_Model_Service_Catalog */
					$catalogService = Mage::getSingleton ( 'viaebay/service_catalog' );
					$catalogService->updateStockById ( $stockItem->getProductId (), $stockItem->getQty () );
					foreach ( Mage::helper ( 'viaebay/backlog' )->getParentIds ( $stockItem->getProductId () ) as $parentId ) {
						$catalogService->updateStockById ( $parentId );
					}
				}
			}
		} catch ( Exception $ex ) {
			Mage::helper ( 'viaebay/log' )->logException ( $ex );
		}
		return $this;
	}
	
	/**
	 * Triggered after order is submitted
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function checkoutSubmitAllAfter(Varien_Event_Observer $observer) {
		Mage::helper ( 'viaebay/log' )->log ( 'checkoutSubmitAllAfter', Zend_Log::DEBUG );
		try {
			$order = $observer->getEvent ()->getOrder ();
			foreach ( $order->getAllItems () as $item ) {
				Mage::helper ( 'viaebay/backlog' )->createBacklog ( $item->getProductId () );
				Mage::helper ( 'viaebay/backlog' )->createParentBacklog ( $item->getProductId () );
			}
		} catch ( Exception $ex ) {
			Mage::helper ( 'viaebay/log' )->logException ( $ex );
		}
		return $this;
	}
	
	/**
	 * Triggered after shipment is saved
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function salesOrderShipmentSaveAfter(Varien_Event_Observer $observer) {
		try {
			$shipment = $observer->getEvent ()->getShipment ();
			Mage::getSingleton ( 'viaebay/service_order' )->updateShipmentStatus ( $shipment );
		} catch ( Exception $e ) {
			Mage::helper ( 'viaebay/log' )->logException ( $e );
		}
		
		return $this;
	}
	
	/**
	 * Triggered after shipment track is saved.
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function salesOrderShipmentTrackSaveAfter(Varien_Event_Observer $observer) {
		try {
			$track = $observer->getEvent ()->getTrack ();
			
			$orderService = Mage::getSingleton ( 'viaebay/service_order' );
			/* @var $orderService VIA_Ebay_Model_Service_Order */
			$orderService->addTrackingNumber ( $track );
		} catch ( Exception $ex ) {
			Mage::helper ( 'viaebay/log' )->logException ( $ex );
		}
		return $this;
	}
	
	/**
	 * Triggered when product is duplicated.
	 *
	 * @param Varien_Event_Observer $observer        	
	 * @return VIA_Ebay_Model_Observer
	 */
	public function productDuplicate(Varien_Event_Observer $observer) {
		$newProduct = $observer->getEvent ()->getNewProduct ();
		$newProduct->setData ( 'viaebay_id', '' );
		$newProduct->setData ( 'viaebay_export', 0 );
		return $this;
	}
}