<?php
class VIA_Ebay_Model_Service_Abstract {
	/**
	 * Process ID.
	 */
	const DEFAULT_PROCESS_ID = 'viaebay';
	
	/**
	 * Used process id.
	 *
	 * @var unknown
	 */
	protected $processID;
	
	/**
	 *
	 * @var VIA_Ebay_Model_Client
	 */
	protected $client;
	
	/**
	 *
	 * @var Mage_Core_Model_App_Emulation
	 */
	protected $appEmulation;
	
	/**
	 * Initial environment when starting emulation.
	 * Used to unset.
	 *
	 * @var array
	 */
	protected $initialEnvironmentInfo;
	
	/**
	 * Lock API.
	 *
	 * @var Mage_Index_Model_Process $indexProcess
	 */
	protected $process;
	
	/**
	 *
	 * @var unknown
	 */
	protected $storeId;
	
	/**
	 *
	 * @var unknown
	 */
	protected $lockFile;
	
	/**
	 *
	 * @var boolean
	 */
	protected $isLocked = false;
	
	/**
	 *
	 * @var VIA_Ebay_Helper_Log
	 */
	protected $log;
	
	/**
	 *
	 * @var Mage_Index_Model_Process
	 */
	protected $productFlatProcess;
	
	/**
	 *
	 * @see Mage_Index_Model_Process
	 * @var string
	 */
	protected $productFlatProcessStatus;
	
	/**
	 *
	 * @var Mage_Index_Model_Process
	 */
	protected $categoryFlatProcess;
	
	/**
	 *
	 * @see Mage_Index_Model_Process
	 * @var string
	 */
	protected $categoryFlatProcessStatus;
	
	/**
	 * Name used to store this process.
	 *
	 * @param string $processID
	 *        	to be used for locking
	 */
	function __construct($processID = null) {
		$this->log = Mage::helper ( 'viaebay/log' );
		$this->processID = $processID;
		$this->storeId = Mage::helper ( 'viaebay/data' )->getVIAStoreId ();
		$this->appEmulation = Mage::getSingleton ( 'core/app_emulation' );
		$this->client = Mage::getSingleton ( 'viaebay/client' );
	}
	
	/**
	 *
	 * @return VIA_Ebay_Model_Client
	 */
	function getClient() {
		return $this->client;
	}
	
	/**
	 * Lock process.
	 *
	 * @throws ErrorException
	 */
	function lock($storeId = null) {
		if ($this->isLocked) {
			return false;
		}
		if (flock ( $this->getLockFile (), LOCK_EX | LOCK_NB )) {
			$this->isLocked = true;
		} else {
			throw new ErrorException ( 'VIA-Connect already running' );
		}
		// Start environment emulation of the specified store
		$this->initialEnvironmentInfo = $this->appEmulation->startEnvironmentEmulation ( $storeId == null ? $this->storeId : $storeId );
		
		return $this->isLocked;
	}
	
	/**
	 * Unlock process.
	 */
	function unlock() {
		if ($this->isLocked) {
			// Stop environment emulation and restore original store
			$this->appEmulation->stopEnvironmentEmulation ( $this->initialEnvironmentInfo );
			flock ( $this->getLockFile (), LOCK_UN );
			$this->isLocked = false;
		}
		
		return $this->isLocked;
	}
	
	/**
	 * Get lock file resource
	 *
	 * @return resource
	 */
	protected function getLockFile() {
		if ($this->processID == null) {
			$id = self::DEFAULT_PROCESS_ID;
		} else {
			$id = $this->processID;
		}
		
		if ($this->lockFile === null) {
			$varDir = Mage::getConfig ()->getVarDir ( 'locks' );
			$file = $varDir . DS . 'sync_process_' . $id . '.lock';
			if (is_file ( $file )) {
				$this->lockFile = fopen ( $file, 'w' );
			} else {
				$this->lockFile = fopen ( $file, 'x' );
			}
			fwrite ( $this->lockFile, date ( 'r' ) );
		}
		return $this->lockFile;
	}
	
	/**
	 * Parse API Date.
	 *
	 * @param string $field        	
	 * @param array $arr        	
	 */
	public function parseDate($field, array $arr) {
		if (array_key_exists ( $field, $arr )) {
			$date = $arr [$field];
			if (strlen ( $date ) && preg_match ( '|/Date\((\d*)\)/|', $date, $match )) {
				if (isset ( $match [1] ) && strlen ( $match [1] )) {
					return date ( 'c', $match [1] / 1000 );
				}
			}
		}
		return '';
	}
	
	/**
	 * Update array diff at key where value at key differs from orig.
	 *
	 * @param array $diff
	 *        	changed values
	 * @param unknown $orig
	 *        	original object
	 * @param unknown $key
	 *        	to check
	 * @param unknown $newValue
	 *        	new value
	 * @return array diff
	 */
	protected function updateDelta(array &$diff, $orig, $key, $newValue) {
		if ($orig == null || ! is_array ( $orig ) || $orig [$key] != $newValue) {
			$diff [$key] = $newValue;
		}
		return $diff;
	}
	
	/**
	 * Update target using values from deleta.
	 *
	 * @param array $target
	 *        	orbject
	 * @param array $delta
	 *        	changed values
	 * @return array target
	 */
	protected function updateObject(array &$target, array $delta) {
		$this->getClient ()->updateObject ( $target, $delta );
		$this->mergeArray ( $target, $delta );
		return $target;
	}
	
	/**
	 * Merge update results into old object
	 *
	 * @param array $target
	 *        	of merge
	 * @param array $newValues
	 *        	to merge
	 * @return array target
	 */
	protected function mergeArray(array &$target, array $newValues) {
		foreach ( $newValues as $key => $value ) {
			$target [$key] = $value;
		}
		return $target;
	}
}
