<?php
class VIA_Ebay_Model_Service_Catalog extends VIA_Ebay_Model_Service_Abstract {
	/**
	 * Array of Magento categories.
	 *
	 * @var Mage_Catalog_Model_Resource_Category_Collection
	 */
	protected $_magentoCategories = null;
	
	/**
	 * Array of VIA catalogs.
	 *
	 * @var array
	 */
	protected $_viaCatalogs = null;
	
	/**
	 *
	 * @var VIA_Ebay_Helper_Data
	 */
	protected $_viaDataHelper = null;
	
	/**
	 *
	 * @var VIA_Ebay_Helper_Product
	 */
	protected $_viaProductHelper = null;
	
	/**
	 *
	 * @var array $_attributeLabels
	 */
	protected $_attributeLabels = null;
	
	/**
	 * Size of listing images.
	 */
	const listingImageSize = 225;
	
	/**
	 * Size of gallery images.
	 */
	const galleryImageSize = 300;
	
	/**
	 * Type id of listing images.
	 */
	const listingImageType = 1;
	
	/**
	 * Type id of gallery images.
	 */
	const galleryImageType = 2;
	
	/**
	 * Default constructor for this class.
	 */
	function __construct() {
		parent::__construct ( "viaebay_catalog_export" );
		$this->_viaDataHelper = Mage::helper ( 'viaebay/data' );
		$this->_viaProductHelper = Mage::helper ( 'viaebay/product' );
		
		$this->_attributeLabels = Mage::getResourceModel ( 'viaebay/attribute' )->getProductAttributeLabels ( $this->_viaDataHelper->getVIAStoreId () );
	}
	
	/**
	 * Run catalog export.
	 *
	 * @return Varien_Object
	 */
	public function export() {
		$result = new Varien_Object ( array (
				'category' => new Varien_Object (),
				'product' => new Varien_Object () 
		) );
		
		try {
			$this->lock ();
			
			$this->log->log ( 'Starting VIAEbay Catalog Export ' . $this->_viaDataHelper->getExtensionVersion (), Zend_Log::INFO );
			
			$this->exportCategories ( $result->category );
			$this->exportProducts ( $result->product );
		} catch ( Exception $e ) {
			$this->log->logException ( $e );
		}
		
		$this->log->log ( 'Completed VIAEbay Catalog Export', Zend_Log::INFO );
		
		$this->unlock ();
		
		return $result;
	}
	
	/**
	 * Run category export.
	 *
	 * @param Varien_Object $result        	
	 * @return Varien_Object
	 */
	public function exportCategories($result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		
		if (! $result->hasData ( 'added' )) {
			$result->added = 0;
		}
		
		if (! $result->hasData ( 'deleted' )) {
			$result->deleted = 0;
		}
		
		if (! $result->hasData ( 'error' )) {
			$result->error = 0;
		}
		
		$seenRemoteCatalogs = array ();
		$seenLocalCategories = array ();
		
		/* @var $resourceModel Mage_Catalog_Model_Resource_Category */
		$resourceModel = Mage::getResourceModel ( 'viaebay/category' );
		
		try {
			$categories = array ();
			
			// Add or update categories
			/* @var $category Mage_Catalog_Model_Category */
			foreach ( $this->getMagentoCategories () as $category ) {
				$seenLocalCategories [$category->getId ()] = $category;
				try {
					$categoryForeignId = $category->getData ( 'viaebay_id' );
					$viaCatalog = $this->getCatalog ( $this->getViaCatalogs (), $categoryForeignId );
					
					$viaCatalogDelta = array ();
					
					$this->updateDelta ( $viaCatalogDelta, $viaCatalog, 'Name', $category->getName () );
					$this->updateDelta ( $viaCatalogDelta, $viaCatalog, 'ForeignId', $category->getId () );
					$this->updateDelta ( $viaCatalogDelta, $viaCatalog, 'IsRootLevel', $category->getLevel () == 1 );
					
					if ($viaCatalog == null) {
						$viaCatalog = $this->getClient ()->saveObject ( 'Catalogs', $viaCatalogDelta );
						$this->addViaCatalog ( $viaCatalog );
						
						// Category is new we need to save the id.
						$category->setData ( 'viaebay_id', $viaCatalog ['Id'] );
						$resourceModel->saveAttribute ( $category, 'viaebay_id' );
						
						$result->added ++;
					} elseif (count ( $viaCatalogDelta ) > 0) {
						$this->updateObject ( $viaCatalog, $viaCatalogDelta );
						$result->updated ++;
					} else {
						$result->unchanged ++;
					}
					
					$seenRemoteCatalogs [$viaCatalog ['Id']] = $viaCatalog;
				} catch ( Exception $ex ) {
					$this->log->logException ( $ex );
					$result->error ++;
				}
			}
			
			// Update category relations
			foreach ( $this->getMagentoCategories () as $category ) {
				try {
					$remoteViaEbayId = $category->getData ( 'viaebay_id' );
					
					$viaChildCatalog = $this->getCatalog ( $this->getViaCatalogs (), $remoteViaEbayId );
					
					$parentCategory = array_key_exists ( $category->getParentId (), $seenLocalCategories ) ? $seenLocalCategories [$category->getParentId ()] : null;
					$viaParentCatalog = null;
					
					// Do we have a parent category
					if ($parentCategory == null || $category->getLevel () <= 1) {
						$this->log->log ( "Root category m" . $category->getId () . " (v" . $viaChildCatalog ['Id'] . ")", Zend_Log::INFO );
					} else {
						$parentViaEbayId = $parentCategory->getData ( 'viaebay_id' );
						
						if ($parentViaEbayId == null) {
							$this->log->log ( "Category not mapped m" . $parentCategory->getId (), Zend_Log::INFO );
							continue;
						}
						
						if (array_key_exists ( $parentViaEbayId, $seenRemoteCatalogs )) {
							$viaParentCatalog = $seenRemoteCatalogs [$parentViaEbayId];
							// Check if connection already exists
							if ($this->getCatalog ( $viaChildCatalog ['ParentCatalogs'], $viaParentCatalog ['Id'] ) != null) {
							} else {
								$this->getClient ()->addLink ( $viaChildCatalog, 'ParentCatalogs', $viaParentCatalog );
								$this->log->log ( "Connected child catalog v" . $viaChildCatalog ['Id'] . " to parent catalog v" . $viaParentCatalog ['Id'], Zend_Log::INFO );
							}
						} else {
							$this->log->log ( "Mapped category not found " . $parentViaEbayId, Zend_Log::WARN );
						}
					}
					
					// Remove unused connections
					if (is_array ( $viaChildCatalog ['ParentCatalogs'] )) {
						foreach ( $viaChildCatalog ['ParentCatalogs'] as $viaParent ) {
							if (isset ( $viaParentCatalog ['Id'] ) && isset ( $viaParent ['Id'] ) && $viaParentCatalog ['Id'] != $viaParent ['Id']) {
								$this->getClient ()->deleteLink ( $viaChildCatalog, 'ParentCatalogs', $viaParent );
								$this->log->log ( "Deleted category v " . $viaCatalog ['Id'] . " from parent v" . $viaParent ['Id'], Zend_Log::INFO );
							}
						}
					}
				} catch ( Exception $ex ) {
					$this->log->logException ( $ex );
					$result->error ++;
				}
			}
			
			$result->deleted += $this->deleteEntityIfIdHasNotBeenSeen ( $this->getViaCatalogs (), $seenRemoteCatalogs );
		} catch ( Exception $e ) {
			$this->log->logException ( $e );
			$result->error ++;
		}
		
		return $result;
	}
	
	/**
	 * Run product export.
	 *
	 * @param Varien_Object $result        	
	 * @return Varien_Object
	 */
	public function exportProducts($result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		
		$result->setData ( array (
				'added' => 0,
				'updated' => 0,
				'deleted' => 0,
				'unchanged' => 0,
				'error' => 0,
				'discount_offer' => new Varien_Object ( array (
						'added' => 0,
						'updated' => 0,
						'deleted' => 0,
						'unchanged' => 0,
						'error' => 0 
				) ),
				'attribute' => new Varien_Object ( array (
						'added' => 0,
						'updated' => 0,
						'deleted' => 0,
						'unchanged' => 0,
						'error' => 0 
				) ),
				'optional_attribute' => new Varien_Object ( array (
						'added' => 0,
						'updated' => 0,
						'deleted' => 0,
						'unchanged' => 0,
						'error' => 0 
				) ),
				'image' => new Varien_Object ( array (
						'added' => 0,
						'updated' => 0,
						'deleted' => 0,
						'unchanged' => 0,
						'error' => 0 
				) ),
				'variation' => new Varien_Object ( array (
						'added' => 0,
						'updated' => 0,
						'deleted' => 0,
						'unchanged' => 0,
						'error' => 0 
				) ),
				'parent' => new Varien_Object ( array (
						'added' => 0,
						'updated' => 0,
						'deleted' => 0,
						'unchanged' => 0,
						'error' => 0 
				) ),
				'category' => new Varien_Object ( array (
						'added' => 0,
						'deleted' => 0,
						'unchanged' => 0,
						'error' => 0 
				) ) 
		) );
		
		$state = new Varien_Object ( array (
				'seenMagentoProducts' => array () 
		) );
		
		$productBacklogs = Mage::getModel ( 'viaebay/backlog_product' )->getCollection ();
		/* @var $productBacklogs VIA_Ebay_Model_Backlog_Product */
		
		try {
			Mage::getSingleton ( 'core/resource_iterator' )->walk ( $productBacklogs->getSelect (), array (
					array (
							$this,
							'exportProductsBacklogCallback' 
					) 
			), array (
					'result' => $result,
					'state' => $state 
			) );
		} catch ( Exception $ex ) {
			$this->log->logException ( $ex );
			$result->error ++;
		}
		
		return $result;
	}
	
	/**
	 * Callback to process a single product.
	 *
	 * @param unknown $args        	
	 */
	public function exportProductsBacklogCallback($args) {
		$result = $args ['result'];
		$state = $args ['state'];
		$seenMagentoProducts = $state->seenMagentoProducts;
		
		$productBacklog = Mage::getModel ( 'viaebay/backlog_product' );
		$productBacklog->setData ( $args ['row'] );
		$productId = $productBacklog->getProductId ();
		
		try {
			if (! isset ( $seenMagentoProducts [$productId] )) {
				$seenMagentoProducts [$productId] = true;
				$state->seenMagentoProducts = $seenMagentoProducts;
				
				$this->exportProductById ( $productId, $result );
			} else {
				$this->log->logToFile ( 'Product m' . $productId . ' already synced', Zend_Log::WARN );
			}
			
			$productBacklog->delete ();
		} catch ( Exception $ex ) {
			$this->log->logException ( $ex, false );
			
			$result->error ++;
			
			$this->log->log ( 'Error syncing product m' . $productId . ": " . $ex->getMessage (), Zend_Log::ERR );
		}
	}
	
	/**
	 * Load single product by id using VIA-Connect store.
	 *
	 * @param unknown $productId        	
	 * @return Mage_Catalog_Model_Product
	 */
	protected function loadProductById($productId) {
		$viaEbayStoreId = $this->_viaDataHelper->getVIAStoreId ();
		
		if ($viaEbayStoreId <= 0) {
			$this->log->log ( "StoreId not set", Zend_Log::CRIT );
			return null;
		}
		
		return Mage::getModel ( 'viaebay/product' )->setStoreId ( $viaEbayStoreId )->load ( $productId );
	}
	
	/**
	 * Export single product by id.
	 *
	 * @param unknown $productId        	
	 */
	public function exportProductById($productId, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		
		/* @var $product Mage_Catalog_Model_Product */
		$product = $this->loadProductById ( $productId );
		
		return $this->exportProduct ( $product, $result );
	}
	
	/**
	 * Export product.
	 *
	 * @param unknown $product        	
	 */
	public function exportProduct(Mage_Catalog_Model_Product $product, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		
		if ($product == null || ! $product->getId () || $product->getData ( 'viaRecursionPrevent' ) === true) {
			// Prevent recursion
			return $result;
		}
		$product->setData ( 'viaRecursionPrevent', true );
		
		if ($product->getStoreId () != $this->_viaDataHelper->getVIAStoreId ()) {
			$this->log->logToFile ( 'Load product using via store.', Zend_Log::WARN );
			$product = $this->loadProductById ( $product->getId () );
			if ($product == null || ! $product->getId ()) {
				$this->log->logToFile ( 'No product found using via store.', Zend_Log::ERR );
				return $result;
			} else if ($product->getStoreId () != $this->_viaDataHelper->getVIAStoreId ()) {
				$this->log->logToFile ( 'Failed loading product using via store. ' . $product->getStoreId (), Zend_Log::ERR );
				return $result;
			}
		}
		
		if (! $product->isVisibleInCatalog () || ! $product->isVisibleInSiteVisibility () || $product->getData ( 'viaebay_export' ) != 1 || $product->getStatus () == Mage_Catalog_Model_Product_Status::STATUS_DISABLED) {
			// Not enabled and exported -> ignore
			try {
				$this->deleteProduct ( $product );
				$result->deleted ++;
			} catch ( Exception $e ) {
				$this->log->log ( 'Could not delete already deleted product m' . $product->getId (), Zend_Log::ERR );
				$result->error ++;
			}
			return $result;
		}
		
		$viaProduct = $this->getViaProduct ( $product->getId () );
		
		switch ($product->getTypeId ()) {
			case Mage_Catalog_Model_Product_Type::TYPE_SIMPLE :
				$productType = 0;
				break;
			case Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE :
				$productType = 1;
				break;
			case Mage_Catalog_Model_Product_Type::TYPE_GROUPED :
				$productType = 2;
				break;
			default :
				$this->log->log ( "Product m" . $product->getId () . " has unsupported type " . $product->getTypeId (), Zend_Log::WARN );
				return $result;
		}
		
		$qtyStock = $this->getQtyStock ( $product );
		
		$viaProductDelta = array ();
		
		$mappedValues = $this->_viaProductHelper->getMappedAttributeValues ( $product );
		
		$autoAcceptPrice = Mage::helper ( 'tax' )->getPrice ( $product, $product->getData ( 'viaebay_auto_accept_price' ), true );
		$autoDeclinePrice = Mage::helper ( 'tax' )->getPrice ( $product, $product->getData ( 'viaebay_auto_decline_price' ), true );
		
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'ExternalProductId', $product->getId () );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'StockAmount', intval ( $qtyStock ) );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'Title', $mappedValues ['Title'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'Price', ( string ) ( float ) $mappedValues ['Price'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'Description', $mappedValues ['Description'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'ShortDescription', $mappedValues ['ShortDescription'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'Ean', $mappedValues ['Ean'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'Isbn', $mappedValues ['Isbn'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'Upc', $mappedValues ['Upc'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'Mpn', $mappedValues ['Mpn'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'Brand', $mappedValues ['Brand'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'UnitQuantity', $mappedValues ['UnitQuantity'] );
		$this->updateDelta ( $viaProductDelta, $viaProduct, 'UnitType', $mappedValues ['UnitType'] );
		
		if ($mappedValues ['Price'] > $autoAcceptPrice && $autoAcceptPrice > $autoDeclinePrice && $autoAcceptPrice != 0 && $autoDeclinePrice != 0) {
			$this->updateDelta ( $viaProductDelta, $viaProduct, 'BestOfferAutoAcceptPrice', ( string ) $autoAcceptPrice );
			$this->updateDelta ( $viaProductDelta, $viaProduct, 'BestOfferMinimumPrice', ( string ) $autoDeclinePrice );
		} else {
			$this->updateDelta ( $viaProductDelta, $viaProduct, 'BestOfferAutoAcceptPrice', NULL );
			$this->updateDelta ( $viaProductDelta, $viaProduct, 'BestOfferMinimumPrice', NULL );
		}
		
		if ($viaProduct == null) {
			$viaProductDelta ['ProductType'] = $productType;
			$viaProductDelta ['ForeignId'] = $product->getId ();
			$viaProduct = $this->getClient ()->saveObject ( 'Products', $viaProductDelta );
			
			// Product is new we need to save the id.
			$product->setData ( 'viaebay_id', $viaProduct ['Id'] );
			$product->getResource ()->saveAttribute ( $product, 'viaebay_id' );
			
			$this->log->log ( "Added product " . $viaProduct ['Title'] . " v" . $viaProduct ['Id'], Zend_Log::INFO );
			$result->added ++;
		} elseif (count ( $viaProductDelta )) {
			$this->updateObject ( $viaProduct, $viaProductDelta );
			$this->log->log ( "Updated product " . $viaProduct ['Title'] . " v" . $viaProduct ['Id'], Zend_Log::INFO );
			$result->updated ++;
		} else {
			$result->unchanged ++;
		}
		
		$exportedProducts = array ();
		if ($product->getTypeId () != Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE) {
			$exportedProducts [] = array (
					'product' => $product,
					'variation' => null,
					'via_product' => $viaProduct,
					'via_variation' => null,
					'values' => $mappedValues 
			);
		}
		
		$this->exportProductAttributes ( $product, $viaProduct, $result->attribute );
		$this->exportOptionalProductAttributes ( $product, $viaProduct, $result->optional_attribute );
		$this->exportProductVariations ( $product, $viaProduct, $exportedProducts, $result->variation );
		$this->exportDiscountOffer ( $viaProduct, $exportedProducts, $result->discount_offer );
		$this->exportProductCategories ( $product, $viaProduct, $result->category );
		$this->exportProductImages ( $product, $viaProduct, $result->image );
		
		$shippingProfileId = $product->getData ( 'viaebay_shipping_profile_id' );
		if ($shippingProfileId) {
			$this->getClient ()->call ( "SetProductProfile", array (
					'productId' => $viaProduct ['Id'] . 'L',
					'type' => 3,
					'profileId' => $shippingProfileId . 'L' 
			) );
		}
		
		return $result;
	}
	
	/**
	 * Export discount offer.
	 *
	 * @param Varien_Object $result        	
	 * @return Varien_Object
	 */
	private function exportDiscountOffer(array $viaProduct, array $exportedProducts, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		
		$seenRemoteDiscountOffers = array ();
		
		foreach ( $exportedProducts as $productArray ) {
			$viaVariation = $productArray ['via_variation'];
			$mappedValues = $productArray ['values'];
			
			if ($mappedValues ['DiscountOffer'] > 0) {
				$variationId = $viaVariation != null ? $viaVariation ['Id'] : null;
				
				$viaDiscountOffer = $this->getByProductId ( $viaProduct ['DiscountOffers'], $viaProduct ['Id'], $variationId );
				$viaDiscountOfferDelta = array ();
				
				$this->updateDelta ( $viaDiscountOfferDelta, $viaDiscountOffer, 'Price', ( string ) ( float ) $mappedValues ['DiscountOffer'] );
				
				if ($viaDiscountOffer == null) {
					$viaDiscountOfferDelta ['ProductId'] = $viaProduct ['Id'];
					if (is_numeric ( $variationId )) {
						$viaDiscountOfferDelta ['ProductVariationId'] = $variationId;
					}
					$viaDiscountOfferDelta ['Type'] = 1; // 1 => Bisher/Ursprünglich
					$viaDiscountOffer = $this->getClient ()->saveObject ( 'DiscountOffers', $viaDiscountOfferDelta );
					$this->log->log ( "Added discount offer " . $viaDiscountOfferDelta ['Type'] . " for product v" . $viaProduct ['Id'] . "to " . $viaDiscountOfferDelta ['Price'], Zend_Log::INFO );
					$result->added ++;
				} elseif (count ( $viaDiscountOfferDelta )) {
					$this->updateObject ( $viaDiscountOffer, $viaDiscountOfferDelta );
					$this->log->log ( "Updated discount offer " . $viaDiscountOffer ['Type'] . " for product v" . $viaProduct ['Id'] . "to " . $viaDiscountOfferDelta ['Price'], Zend_Log::INFO );
					$result->updated ++;
				} else {
					// Nothing to do ;-)
					$result->unchanged ++;
				}
				
				$seenRemoteDiscountOffers [$viaDiscountOffer ['Id']] = $viaDiscountOffer;
			}
		}
		
		if (is_array ( $viaProduct ['DiscountOffers'] )) {
			$result->deleted += $this->deleteEntityIfIdHasNotBeenSeen ( $viaProduct ['DiscountOffers'], $seenRemoteDiscountOffers );
		}
		return $result;
	}
	
	/**
	 * Export product attributes.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param array $viaProduct        	
	 * @return Varien_Object
	 */
	private function exportProductAttributes(Mage_Catalog_Model_Product $product, array $viaProduct, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		
		$seenRemoteAttributes = array ();
		
		foreach ( $product->getAttributes () as $attribute ) {
			if ($attribute->getViaebayAttribute () != 'CustomAttribute') {
				continue;
			}
			
			$productValue = $product->getData ( $attribute->getAttributeCode () );
			
			if ($attribute->isValueEmpty ( $productValue ) || empty ( $productValue )) {
				continue;
			}
			
			$frontendValue = $attribute->getFrontend ()->getValue ( $product );
			
			if (empty ( $frontendValue )) {
				continue;
			}
			
			$storeLabel = $attribute->getStoreLabel ( $this->_viaDataHelper->getVIAStoreId () );
			
			if (strlen ( $storeLabel ) <= 0) {
				$storeLabel = $attribute->getFrontendLabel ();
			}
			
			if (strlen ( $storeLabel ) <= 0) {
				throw new Exception ( "Cannot get store label for " . $attribute->getAttributeCode () . ' and product m' . $product->getId () );
			}
			
			$viaProductSpecific = $this->getByName ( $viaProduct ['ProductSpecifics'], $storeLabel );
			
			$viaProductSpecificsDelta = array ();
			
			$this->updateDelta ( $viaProductSpecificsDelta, $viaProductSpecific, 'Value', $frontendValue );
			
			if ($viaProductSpecific == null) {
				$viaProductSpecificsDelta ['ProductId'] = $viaProduct ['Id'];
				$viaProductSpecificsDelta ['Name'] = $storeLabel;
				$viaProductSpecific = $this->getClient ()->saveObject ( 'ProductSpecifics', $viaProductSpecificsDelta );
				$this->log->log ( "Added product specifics " . $viaProductSpecificsDelta ['Name'] . " to product v" . $viaProduct ['Id'], Zend_Log::INFO );
				$result->added ++;
			} elseif (count ( $viaProductSpecificsDelta )) {
				$this->updateObject ( $viaProductSpecific, $viaProductSpecificsDelta );
				$this->log->log ( "Updated product specifics " . $viaProductSpecific ['Name'] . " for product v" . $viaProduct ['Id'], Zend_Log::INFO );
				$result->updated ++;
			} else {
				// Nothing to do ;-)
				$result->unchanged ++;
			}
			
			$seenRemoteAttributes [$viaProductSpecific ['Id']] = $viaProductSpecific;
		}
		
		if (is_array ( $viaProduct ['ProductSpecifics'] )) {
			$result->deleted += $this->deleteEntityIfIdHasNotBeenSeen ( $viaProduct ['ProductSpecifics'], $seenRemoteAttributes );
		}
		
		return $result;
	}
	
	/**
	 * Export optional product attributes.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param array $viaProduct        	
	 * @return Varien_Object
	 */
	private function exportOptionalProductAttributes(Mage_Catalog_Model_Product $product, array $viaProduct, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		$seenRemoteAttributes = array ();
		
		foreach ( $product->getAttributes () as $attribute ) {
			if ($attribute->getViaebayAttribute () != 'OptionalAttribute') {
				continue;
			}
			
			$productValue = $product->getData ( $attribute->getAttributeCode () );
			
			if ($attribute->isValueEmpty ( $productValue ) || empty ( $productValue )) {
				continue;
			}
			
			$frontendValue = $attribute->getFrontend ()->getValue ( $product );
			
			if (empty ( $frontendValue )) {
				continue;
			}
			
			$storeLabel = $attribute->getStoreLabel ( $this->_viaDataHelper->getVIAStoreId () );
			
			if (strlen ( $storeLabel ) <= 0) {
				$storeLabel = $attribute->getFrontendLabel ();
			}
			
			if (strlen ( $storeLabel ) <= 0) {
				throw new Exception ( "Cannot get store label for " . $attribute->getAttributeCode () . ' and product m' . $product->getId () );
			}
			
			$viaOptionalAttribute = $this->getByName ( $viaProduct ['OptionalProductAttributes'], $storeLabel );
			
			$viaOptionalAttributeDelta = array ();
			
			$this->updateDelta ( $viaOptionalAttributeDelta, $viaOptionalAttribute, 'Value', $frontendValue );
			
			if ($viaOptionalAttribute == null) {
				$viaOptionalAttributeDelta ['ProductId'] = $viaProduct ['Id'];
				$viaOptionalAttributeDelta ['Name'] = $storeLabel;
				$viaOptionalAttribute = $this->getClient ()->saveObject ( 'OptionalProductAttributes', $viaOptionalAttributeDelta );
				$this->log->log ( "Added optional attribute " . $viaOptionalAttributeDelta ['Name'] . " to product v" . $viaProduct ['Id'], Zend_Log::INFO );
				$result->added ++;
			} elseif (count ( $viaOptionalAttributeDelta )) {
				$this->updateObject ( $viaOptionalAttribute, $viaOptionalAttributeDelta );
				$this->log->log ( "Updated optional attribute " . $viaOptionalAttribute ['Name'] . " for product v" . $viaProduct ['Id'], Zend_Log::INFO );
				$result->updated ++;
			} else {
				// Nothing to do ;-)
				$result->unchanged ++;
			}
			
			$seenRemoteAttributes [$viaOptionalAttribute ['Id']] = $viaOptionalAttribute;
		}
		
		if (is_array ( $viaProduct ['OptionalProductAttributes'] )) {
			$result->deleted += $this->deleteEntityIfIdHasNotBeenSeen ( $viaProduct ['OptionalProductAttributes'], $seenRemoteAttributes );
		}
		
		return $result;
	}
	
	/**
	 * Export product images.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param array $viaProduct        	
	 * @return array
	 */
	private function exportProductImages(Mage_Catalog_Model_Product $product, array $viaProduct, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		$seenRemoteImages = array ();
		
		$forceHTTPMedia = Mage::getStoreConfigFlag ( 'viaebay/extended/force_http_media' );
		
		$images = $product->getMediaGalleryImages ();
		if ($images != null) {
			$imagePos = 3;
			
			foreach ( $images as $image ) {
				$smallOrImage = false;
				
				if ($image->getDisabled ()) {
					continue;
				}
				
				if ($product->getData ( 'small_image' ) == $image->getFile ()) {
					$imageUrl = $image->getUrl ();
					if ($forceHTTPMedia) {
						$imageUrl = str_replace ( 'https://', 'http://', $imageUrl );
					}
					$viaProductImage = $this->exportProductImage ( $product, $viaProduct, $imageUrl, self::listingImageType, $result );
					$seenRemoteImages [$viaProductImage ['Id']] = true;
					$smallOrImage = true;
				}
				
				if ($product->getData ( 'image' ) == $image->getFile ()) {
					$imageUrl = $image->getUrl ();
					if ($forceHTTPMedia) {
						$imageUrl = str_replace ( 'https://', 'http://', $imageUrl );
					}
					$viaProductImage = $this->exportProductImage ( $product, $viaProduct, $imageUrl, self::galleryImageType, $result );
					$seenRemoteImages [$viaProductImage ['Id']] = true;
					$smallOrImage = true;
				}
				
				if ($smallOrImage) {
					continue;
				}
				
				if ($imagePos <= 12) {
					$imageUrl = $image->getUrl ();
					if ($forceHTTPMedia) {
						$imageUrl = str_replace ( 'https://', 'http://', $imageUrl );
					}
					$viaProductImage = $this->exportProductImage ( $product, $viaProduct, $imageUrl, $imagePos ++, $result );
					$seenRemoteImages [$viaProductImage ['Id']] = true;
				} else {
					$this->log->log ( 'More than 10 images for product v' . $viaProduct ['Id'], Zend_Log::WARN );
				}
			}
		}
		
		if (is_array ( $viaProduct ['ProductImages'] )) {
			foreach ( $viaProduct ['ProductImages'] as $viaProductImage ) {
				if (isset ( $viaProductImage ['Id'] ) && (! isset ( $seenRemoteImages [$viaProductImage ['Id']] ) || $seenRemoteImages [$viaProductImage ['Id']] == false)) {
					$this->getClient ()->deleteObject ( $viaProductImage );
					$this->log->log ( "Deleted product image v" . $viaProductImage ['Id'] . " " . $viaProductImage ['ImageUrl'], Zend_Log::INFO );
					$result->deleted ++;
				}
			}
		}
		return $result;
	}
	
	/**
	 * Export single product image.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param array $viaProduct        	
	 * @param unknown $imageUrl        	
	 * @param unknown $type        	
	 * @param array $result        	
	 * @return array
	 */
	private function exportProductImage(Mage_Catalog_Model_Product $product, array $viaProduct, $imageUrl, $type, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		$viaProductImage = $this->getProductImage ( $viaProduct ['ProductImages'], $imageUrl, $type );
		
		$viaProductImageDelta = array ();
		
		$this->updateDelta ( $viaProductImageDelta, $viaProductImage, 'ImageUrl', $imageUrl );
		$this->updateDelta ( $viaProductImageDelta, $viaProductImage, 'ProductId', $viaProduct ['Id'] );
		$this->updateDelta ( $viaProductImageDelta, $viaProductImage, 'Type', $type );
		
		if ($viaProductImage == null) {
			$viaProductImage = $this->getClient ()->saveObject ( 'ProductImages', $viaProductImageDelta );
			$this->log->log ( "Added product image v" . $viaProductImage ['Id'] . " to product v" . $viaProduct ['Id'] . " using url " . $viaProductImage ['ImageUrl'] . ' with type ' . $viaProductImage ['Type'], Zend_Log::INFO );
			$result->added ++;
		} elseif (count ( $viaProductImageDelta )) {
			$this->updateObject ( $viaProductImage, $viaProductImageDelta );
			$this->log->log ( "Updated product image v" . $viaProductImage ['Id'] . " using url " . $viaProductImage ['ImageUrl'] . ' with type ' . $viaProductImage ['Type'], Zend_Log::INFO );
			$result->updated ++;
		} else {
			// Nothing to do
			$result->unchanged ++;
		}
		
		return $viaProductImage;
	}
	
	/**
	 * Export product images.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param array $viaProduct        	
	 * @return array
	 */
	private function exportProductVariationPictures(Mage_Catalog_Model_Product $product, array $viaProductVariation, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		$seenRemoteImages = array ();
		
		$forceHTTPMedia = Mage::getStoreConfigFlag ( 'viaebay/extended/force_http_media' );
		
		$imageArray = array ();
		
		$images = $product->getMediaGalleryImages ();
		if ($images != null) {
			foreach ( $images as $image ) {
				if ($image->getDisabled ()) {
					continue;
				}
				
				if ($product->getData ( 'small_image' ) == $image->getFile ()) {
					array_unshift ( $imageArray, $image->getUrl () );
					continue;
				}
				
				if ($product->getData ( 'image' ) == $image->getFile ()) {
					array_unshift ( $imageArray, $image->getUrl () );
					continue;
				}
				
				if (count ( $imageArray ) <= 12) {
					$imageArray [] = $image->getUrl ();
				} else {
					$this->log->log ( 'More than 10 images for product v' . $viaProductVariation ['Id'], Zend_Log::WARN );
				}
			}
		}
		
		$imagePos = 1;
		foreach ( $imageArray as $url ) {
			if ($forceHTTPMedia) {
				$url = str_replace ( 'https://', 'http://', $url );
			}
			
			$viaProductImage = $this->getProductPictureByImagePosition ( $viaProductVariation ['ProductVariationPictures'], $imagePos );
			
			$viaProductImageDelta = array ();
			
			$this->updateDelta ( $viaProductImageDelta, $viaProductImage, 'ImageUrl', $url );
			$this->updateDelta ( $viaProductImageDelta, $viaProductImage, 'ImagePosition', $imagePos );
			
			if ($viaProductImage == null) {
				$viaProductImageDelta ['ProductVariationId'] = $viaProductVariation ['Id'];
				$viaProductImage = $this->getClient ()->saveObject ( 'ProductVariationPictures', $viaProductImageDelta );
				$this->log->log ( "Added product variation image v" . $viaProductImage ['Id'] . " to product v" . $viaProductVariation ['Id'] . " using url " . $viaProductImage ['ImageUrl'] . ' with type ' . $viaProductImage ['Type'], Zend_Log::INFO );
				$result->added ++;
			} elseif (count ( $viaProductImageDelta )) {
				$this->updateObject ( $viaProductImage, $viaProductImageDelta );
				$this->log->log ( "Updated product variation image v" . $viaProductImage ['Id'] . " using url " . $viaProductImage ['ImageUrl'] . ' with type ' . $viaProductImage ['Type'], Zend_Log::INFO );
				$result->updated ++;
			} else {
				// Nothing to do
				$result->unchanged ++;
			}
			
			$seenRemoteImages [$viaProductImage ['Id']] = true;
			$imagePos ++;
		}
		
		if (is_array ( $viaProductVariation ['ProductVariationPictures'] )) {
			foreach ( $viaProductVariation ['ProductVariationPictures'] as $viaProductImage ) {
				if (isset ( $viaProductImage ['Id'] ) && (! isset ( $seenRemoteImages [$viaProductImage ['Id']] ) || $seenRemoteImages [$viaProductImage ['Id']] == false)) {
					$this->getClient ()->deleteObject ( $viaProductImage );
					$this->log->log ( "Deleted product image v" . $viaProductImage ['Id'] . " " . $viaProductImage ['ImageUrl'], Zend_Log::INFO );
					$result->deleted ++;
				}
			}
		}
		return $result;
	}
	
	/**
	 * Export product variations.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param array $viaProduct        	
	 * @return array:
	 */
	private function exportProductVariations(Mage_Catalog_Model_Product $product, array $viaProduct, &$exportedProducts, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		$typeInstance = $product->getTypeInstance ( false );
		
		if ($product->getTypeId () != Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE && ! ($typeInstance instanceof Mage_Catalog_Model_Product_Type_Configurable)) {
			return $result;
		}
		
		$productAttributeOptions = $typeInstance->getConfigurableAttributeCollection ( $product )->orderByPosition ();
		
		$this->log->logToFile ( sprintf ( "Variation attribute storeId %s", $productAttributeOptions->getStoreId () ), Zend_Log::DEBUG );
		
		$simpleProducts = $typeInstance->getUsedProductCollection ( $product )->addAttributeToSelect ( '*' )->addFilterByRequiredOptions ();
		
		$seenRemoteVariations = array ();
		
		foreach ( $simpleProducts as $simpleProduct ) {
			/* @var $simpleProduct Mage_Catalog_Model_Product */
			
			if ($simpleProduct->getStatus () == Mage_Catalog_Model_Product_Status::STATUS_DISABLED) {
				continue;
			}
			
			/* @var $simpleProduct Mage_Catalog_Model_Product */
			// $simpleProduct = Mage::getModel ( 'catalog/product' )->load ( $simpleProduct->getId () );
			$simpleProduct = $this->loadProductById ( $simpleProduct->getId () );
			
			$mappedValues = $this->_viaProductHelper->getMappedAttributeValues ( $simpleProduct, $product );
			
			$viaProductVariation = $this->getByForeignId ( $viaProduct ['ProductVariations'], $simpleProduct->getId () );
			if ($viaProductVariation == null) {
				// Fix missing ForeignId from API.
				$viaProductVariation = $this->getBySku ( $viaProduct ['ProductVariations'], $simpleProduct->getSku () );
			}
			
			$qtyStock = $this->getQtyStock ( $simpleProduct );
			
			$viaProductVariationDelta = array ();
			
			$this->updateDelta ( $viaProductVariationDelta, $viaProductVariation, 'ExternalProductId', $simpleProduct->getSku () );
			$this->updateDelta ( $viaProductVariationDelta, $viaProductVariation, 'Ean', $mappedValues ['Ean'] );
			$this->updateDelta ( $viaProductVariationDelta, $viaProductVariation, 'Isbn', $mappedValues ['Isbn'] );
			$this->updateDelta ( $viaProductVariationDelta, $viaProductVariation, 'Upc', $mappedValues ['Upc'] );
			$this->updateDelta ( $viaProductVariationDelta, $viaProductVariation, 'Mpn', $mappedValues ['Mpn'] );
			$this->updateDelta ( $viaProductVariationDelta, $viaProductVariation, 'Price', ( string ) ( float ) $mappedValues ['Price'] );
			$this->updateDelta ( $viaProductVariationDelta, $viaProductVariation, 'StockAmount', intval ( $qtyStock > 0 ? $qtyStock : 0 ) );
			$this->updateDelta ( $viaProductVariationDelta, $viaProductVariation, 'ForeignId', $simpleProduct->getId () );
			
			if ($viaProductVariation == null) {
				$viaProductVariationDelta ['ProductId'] = $viaProduct ['Id'];
				$viaProductVariationDelta ['Sku'] = $simpleProduct->getSku ();
				
				$viaProductVariation = $this->getClient ()->saveObject ( 'ProductVariations', $viaProductVariationDelta );
				
				$this->log->log ( "Added product variation " . $viaProductVariation ['Id'] . " to product v" . $viaProduct ['Id'], Zend_Log::INFO );
				$result->added ++;
			} elseif (count ( $viaProductVariationDelta )) {
				$this->updateObject ( $viaProductVariation, $viaProductVariationDelta );
				$this->log->log ( "Updated product variation " . $viaProductVariation ['Id'] . " for product v" . $viaProduct ['Id'], Zend_Log::INFO );
				$result->updated ++;
			} else {
				$result->unchanged ++;
			}
			
			// Product is new we need to save the id.
			$variantIds = unserialize ( $simpleProduct->getData ( 'viaebay_variant_ids' ) );
			if (! is_array ( $variantIds )) {
				$variantIds = array ();
			}
			
			if (! isset ( $variantIds [$product->getId ()] )) {
				$variantIds [$product->getId ()] = array (
						'productId' => $product->getId (),
						'via_productId' => $viaProductVariation ['ProductId'],
						'variationId' => $simpleProduct->getId (),
						'via_variationId' => $viaProductVariation ['Id'] 
				);
				
				$simpleProduct->setData ( 'viaebay_variant_ids', serialize ( $variantIds ) );
				$simpleProduct->getResource ()->saveAttribute ( $simpleProduct, 'viaebay_variant_ids' );
			}
			
			$seenRemoteVariations [$viaProductVariation ['Id']] = $simpleProduct->getId ();
			
			$this->exportProductVariationSpecifics ( $simpleProduct, $viaProductVariation, $productAttributeOptions, $result->productVariationSpecifics );
			$this->exportProductVariationPictures ( $simpleProduct, $viaProductVariation, $result->productVariationPictures );
			
			$exportedProducts [] = array (
					'product' => $product,
					'variation' => $simpleProduct,
					'via_product' => $viaProduct,
					'via_variation' => $viaProductVariation,
					'values' => $mappedValues 
			);
		}
		
		if (is_array ( $viaProduct ['ProductVariations'] )) {
			foreach ( $viaProduct ['ProductVariations'] as $viaProductVariation ) {
				if (isset ( $viaProductVariation ['Id'] ) && (! isset ( $seenRemoteVariations [$viaProductVariation ['Id']] ))) {
					// Variations cannot be deleted from ebay after a sale. We set StockAmount to 0 instead of deleting.
					if ($viaProductVariation ['StockAmount'] > 0) {
						$this->updateObject ( $viaProductVariation, array (
								'StockAmount' => 0 
						) );
					}
					
					// Product is new we need to save the id.
					$variantIds = unserialize ( $product->getData ( 'viaebay_variant_ids' ) );
					if (is_array ( $variantIds )) {
						unset ( $variantIds [$product->getId ()] );
						$product->setData ( 'viaebay_variant_ids', serialize ( $variantIds ) );
						$product->getResource ()->saveAttribute ( $product, 'viaebay_variant_ids' );
					}
					
					$result->deleted ++;
				}
			}
		}
		return $result;
	}
	
	/**
	 * Export product variation specifics.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param array $viaProductVariation        	
	 * @param array $productAttributeOptions        	
	 * @return array
	 */
	private function exportProductVariationSpecifics(Mage_Catalog_Model_Product $product, array $viaProductVariation, Mage_Catalog_Model_Resource_Product_Type_Configurable_Attribute_Collection $configurableAttributes, $result) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		$seenRemoteVariationsSpecifics = array ();
		
		foreach ( $configurableAttributes as $configurableAttribute ) {
			/* @var $attributeOptions Mage_Catalog_Model_Resource_Product_Type_Configurable_Attribute */
			
			$attributeCode = $configurableAttribute->getProductAttribute ()->getAttributeCode ();
			
			if ($configurableAttribute->getUseDefault ()) {
				$this->log->logToFile ( "Using default attribute label", Zend_Log::DEBUG );
				$name = $this->_attributeLabels [$attributeCode];
			} else {
				$this->log->logToFile ( "Using product attribute label", Zend_Log::DEBUG );
				$name = $configurableAttribute->getLabel ();
			}
			$this->log->logToFile ( "Used attribute label is " . $name, Zend_Log::DEBUG );
			
			$viaProductVariationSpecific = $this->getByName ( $viaProductVariation ['ProductVariationSpecifics'], $name );
			
			$viaProductVariationSpecificDelta = array ();
			
			$this->updateDelta ( $viaProductVariationSpecificDelta, $viaProductVariationSpecific, 'Value', $product->getAttributeText ( $attributeCode ) );
			
			if ($viaProductVariationSpecific == null) {
				$viaProductVariationSpecificDelta ['ProductVariationId'] = $viaProductVariation ['Id'];
				$viaProductVariationSpecificDelta ['Name'] = $name;
				$viaProductVariationSpecific = $this->getClient ()->saveObject ( 'ProductVariationSpecifics', $viaProductVariationSpecificDelta );
				$this->log->log ( "Added product variation specific " . $viaProductVariationSpecificDelta ['Name'] . " to product variation v" . $viaProductVariation ['Id'], Zend_Log::INFO );
				$result->added ++;
			} elseif (count ( $viaProductVariationSpecificDelta )) {
				$this->updateObject ( $viaProductVariationSpecific, $viaProductVariationSpecificDelta );
				$this->log->log ( "Updated product variation specific " . $viaProductVariationSpecificDelta ['Name'] . " (v" . $viaProductVariationSpecific ['Id'] . ") for product variation v" . $viaProductVariation ['Id'], Zend_Log::INFO );
				$result->updated ++;
			} else {
				// Nothing to do
				$result->unchanged ++;
			}
			
			$seenRemoteVariationsSpecifics [$viaProductVariationSpecific ['Id']] = $viaProductVariationSpecific;
		}
		
		if (is_array ( $viaProductVariation ['ProductVariationSpecifics'] )) {
			$result->deleted += $this->deleteEntityIfIdHasNotBeenSeen ( $viaProductVariation ['ProductVariationSpecifics'], $seenRemoteVariationsSpecifics );
		}
		
		return $result;
	}
	
	/**
	 * Export product categories.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param array $viaProduct        	
	 * @return array
	 */
	private function exportProductCategories(Mage_Catalog_Model_Product $product, array $viaProduct, $result = null) {
		if ($result == null) {
			$result = new Varien_Object ();
		}
		
		// Fetch all categories for product that are already synced with via
		$viaCategories = array ();
		$collection = $product->getCategoryCollection ()->addAttributeToFilter ( 'viaebay_id', array (
				'notnull' => true 
		) )->addAttributeToSelect ( 'viaebay_id' );
		
		foreach ( $collection as $magentoCategory ) {
			// Somehow attributes are in the result but not the object - argh
			$magentoCategory = Mage::getModel ( 'viaebay/category', array (
					'disable_flat' => true 
			) )->load ( $magentoCategory->getId () );
			/* @var $magentoCategory Mage_Catalog_Model_Category */
			$viaCatalogId = $magentoCategory->getData ( 'viaebay_id' );
			if ($viaCatalogId != null) {
				$viaCategory = $this->getCatalog ( $this->getViaCatalogs (), $viaCatalogId );
				if ($viaCategory != null) {
					$viaCategories [] = $viaCategory;
				}
			}
		}
		
		$seenRemoteCategories = array ();
		// Add products to categories
		foreach ( $viaCategories as $viaCatalog ) {
			if ($this->getProduct ( $viaProduct ['Catalogs'], $viaCatalog ['Id'] ) == null) {
				$this->log->log ( "Add product m" . $product->getId () . " (v" . $viaProduct ['Id'] . ") to catalog v" . $viaCatalog ['Id'], Zend_Log::INFO );
				$this->getClient ()->addLink ( $viaProduct, 'Catalogs', $viaCatalog );
				$result->added ++;
			} else {
				$result->unchanged ++;
			}
			
			$seenRemoteCategories [$viaCatalog ['Id']] = $viaCatalog;
		}
		
		// Remove products from old categories
		if (is_array ( $viaProduct ['Catalogs'] )) {
			foreach ( $viaProduct ['Catalogs'] as $viaCatalog ) {
				if (isset ( $viaCatalog ['Id'] ) && (! isset ( $seenRemoteCategories [$viaCatalog ['Id']] ) || $seenRemoteCategories [$viaCatalog ['Id']] == null)) {
					$this->log->log ( "Delete product m" . $product->getId () . "(v" . $viaProduct ['Id'] . ") from catalog v" . $viaCatalog ['Id'], Zend_Log::INFO );
					$this->getClient ()->deleteLink ( $viaProduct, 'Catalogs', $viaCatalog );
					$result->deleted ++;
				}
			}
		}
		
		return $result;
	}
	
	/**
	 * Delete product.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 */
	public function deleteProduct(Mage_Catalog_Model_Product $product) {
		$viaebayId = $product->getData ( 'viaebay_id' );
		if ($viaebayId != null) {
			try {
				if ($product->getTypeId () == Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE) {
					$this->deleteProductAsVariation ( $product );
				}
				
				$this->getClient ()->deleteObject ( 'Products', $viaebayId );
			} catch ( Exception $e ) {
				$this->log->logException ( $e );
			}
			
			// Product is new we need to save the id.
			$product->unsetData ( 'viaebay_id' );
			$product->getResource ()->saveAttribute ( $product, 'viaebay_id' );
		}
	}
	
	/**
	 * Delete variations by product.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 */
	public function deleteProductAsVariation(Mage_Catalog_Model_Product $product) {
		$viaebayId = $product->getData ( 'viaebay_id' );
		
		// Set StockAmount to 0 as Variations cannot be deleted on ebay.
		foreach ( $this->getClient ()->loadProductVariations ( "ProductId eq " . $viaebayId, null ) as $viaProductVariation ) {
			try {
				$this->updateObject ( $viaProductVariation, array (
						'StockAmount' => 0 
				) );
				
				$simpleProduct = Mage::getModel ( 'viaebay/product' )->load ( $viaProductVariation ['ForeignId'] );
				if ($simpleProduct->getId () && $simpleProduct->getData ( 'viaebay_variant_ids' ) != null) {
					$simpleProduct->unsetData ( 'viaebay_variant_ids' );
					$simpleProduct->getResource ()->saveAttribute ( $simpleProduct, 'viaebay_variant_ids' );
				}
			} catch ( Exception $e ) {
				$this->log->logException ( $e );
			}
		}
	}
	
	/**
	 * Get stock qty for given product.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param Mage_CatalogInventory_Model_Stock_Item $stockItem        	
	 */
	private function getQtyStock(Mage_Catalog_Model_Product $product, $stockItem = null) {
		/* @var $qtyStock Mage_CatalogInventory_Model_Stock_Item */
		if ($stockItem == null) {
			$stockItem = Mage::getModel ( 'cataloginventory/stock_item' )->loadByProduct ( $product );
		}
		
		$qtyStock = 0;
		
		if ($stockItem->getManageStock ()) {
			if ($stockItem->getIsInStock () && $stockItem->getQty () > 0) {
				$qtyStock = $stockItem->getQty ();
			}
		} else {
			$qtyStock = Mage::getStoreConfig ( 'viaebay/catalog/default_stock' );
		}
		
		if ($qtyStock < 0) {
			$qtyStock = 0;
		}
		
		return $qtyStock;
	}
	
	/**
	 * Update stock by product id.
	 *
	 * @param int $product        	
	 * @param string $qty        	
	 * @return NULL
	 */
	public function updateStockById($productId, $qtyStock = null) {
		$product = Mage::getModel ( 'viaebay/product' )->load ( $productId );
		/* @var $product Mage_Catalog_Model_Product */
		$this->updateStock ( $product, $qtyStock );
	}
	
	/**
	 * Update stock by object.
	 *
	 * @param Mage_Catalog_Model_Product $product        	
	 * @param string $qty        	
	 * @return NULL
	 */
	public function updateStock(Mage_Catalog_Model_Product $product, $qtyStock = null) {
		if ($qtyStock === null) {
			$qtyStock = $this->getQtyStock ( $product );
		}
		
		$viaebayId = $product->getData ( 'viaebay_id' );
		$viaebayVariantIds = unserialize ( $product->getData ( 'viaebay_variant_ids' ) );
		
		try {
			if ($product->getTypeId () == Mage_Catalog_Model_Product_Type::TYPE_SIMPLE && $viaebayId) {
				$params = array (
						'productId' => $viaebayId . 'L',
						'stockAmount' => intval ( $qtyStock > 0 ? $qtyStock : 0 ) 
				);
				
				$this->getClient ()->call ( "ReviseInventoryStatus", $params );
			}
			
			if (is_array ( $viaebayVariantIds )) {
				foreach ( $viaebayVariantIds as $productId => $variantId ) {
					try {
						$params = array (
								'productId' => $variantId ['via_productId'] . 'L',
								'productVariationId' => $variantId ['via_variationId'] . 'L',
								'stockAmount' => intval ( $qtyStock > 0 ? $qtyStock : 0 ) 
						);
						
						$this->getClient ()->call ( "ReviseInventoryStatus", $params );
					} catch ( Exception $e ) {
						$this->log->logException ( $e );
					}
				}
			}
		} catch ( Exception $e ) {
			$this->log->logException ( $e );
		}
	}
	
	/**
	 * Syncronize price and stock.
	 */
	public function syncPriceAndStock() {
		$result = new Varien_Object ( array (
				'product' => 0,
				'variation' => 0,
				'error' => 0 
		) );
		
		$viaEbayStoreId = $this->_viaDataHelper->getVIAStoreId ();
		
		$collection = Mage::getModel ( 'viaebay/product' )->setStoreId ( $viaEbayStoreId )->getCollection ();
		$collection->addAttributeToSelect ( '*' );
		$collection->addFieldToFilter ( 'viaebay_id', array (
				'notnull' => true 
		) );
		
		Mage::getSingleton ( 'core/resource_iterator' )->walk ( $collection->getSelect (), array (
				array (
						$this,
						'syncPriceAndStockSimpleProductCallback' 
				) 
		), array (
				'result' => $result,
				'viaEbayStoreId' => $viaEbayStoreId 
		) );
		
		$collection = Mage::getModel ( 'viaebay/product' )->setStoreId ( $viaEbayStoreId )->getCollection ();
		$collection->addAttributeToSelect ( '*' );
		$collection->addFieldToFilter ( 'viaebay_variant_ids', array (
				'notnull' => true 
		) );
		
		Mage::getSingleton ( 'core/resource_iterator' )->walk ( $collection->getSelect (), array (
				array (
						$this,
						'syncPriceAndStockConfigurableProductCallback' 
				) 
		), array (
				'result' => $result,
				'viaEbayStoreId' => $viaEbayStoreId 
		) );
		
		return $result;
	}
	
	/**
	 * Callback to sync stock and price for simple products.
	 * see syncPriceAndStock.
	 *
	 * @param array $args        	
	 */
	public function syncPriceAndStockSimpleProductCallback($args) {
		$result = $args ['result'];
		$viaEbayStoreId = $args ['viaEbayStoreId'];
		
		$productId = $args ['row'] ['entity_id'];
		/* @var $product Mage_Catalog_Model_Product */
		$product = Mage::getModel ( 'viaebay/product' )->setStoreId ( $viaEbayStoreId )->load ( $productId );
		
		if ($product->getTypeId () != Mage_Catalog_Model_Product_Type::TYPE_SIMPLE) {
			return;
		}
		
		$mappedAttributes = $this->_viaProductHelper->getMappedAttributeValues ( $product );
		
		$qtyStock = Mage::getModel ( 'cataloginventory/stock_item' )->loadByProduct ( $product )->getQty ();
		/* @var $qtyStock Mage_CatalogInventory_Model_Stock_Item */
		
		$viaebayId = $product->getData ( 'viaebay_id' );
		
		if ($viaebayId) {
			try {
				$request = array (
						'productId' => $product->getData ( 'viaebay_id' ) . 'L',
						'stockAmount' => intval ( $qtyStock > 0 ? $qtyStock : 0 ),
						'price' => $mappedAttributes ['Price'] . 'm' 
				);
				
				if ($mappedAttributes ['DiscountOffer']) {
					$request ['DiscountOffer'] = $mappedAttributes ['DiscountOffer'] . 'm';
				}
				
				$this->getClient ()->call ( "ReviseInventoryStatus", $request );
				
				$result->product ++;
			} catch ( Exception $e ) {
				$this->log->logException ( $e );
				$result->error ++;
			}
		}
	}
	
	/**
	 * Callback to sync stock and price for configurable products.
	 * see syncPriceAndStock.
	 *
	 * @param array $args        	
	 */
	public function syncPriceAndStockConfigurableProductCallback($args) {
		$result = $args ['result'];
		$viaEbayStoreId = $args ['viaEbayStoreId'];
		
		$productId = $args ['row'] ['entity_id'];
		/* @var $product Mage_Catalog_Model_Product */
		$product = Mage::getModel ( 'viaebay/product' )->setStoreId ( $viaEbayStoreId )->load ( $productId );
		
		// Simple products get all variant ids assigned they belong to
		if ($product->getTypeId () != Mage_Catalog_Model_Product_Type::TYPE_SIMPLE) {
			return;
		}
		
		$viaebayVariantIds = unserialize ( $product->getData ( 'viaebay_variant_ids' ) );
		
		if (is_array ( $viaebayVariantIds )) {
			foreach ( $viaebayVariantIds as $productId => $variantId ) {
				$productVariant = Mage::getModel ( 'viaebay/product' )->setStoreId ( $viaEbayStoreId )->load ( $variantId ['variationId'] );
				/* @var $product Mage_Catalog_Model_Product */
				
				$mappedAttributes = $this->_viaProductHelper->getMappedAttributeValues ( $productVariant, $product );
				
				$qtyStock = Mage::getModel ( 'cataloginventory/stock_item' )->loadByProduct ( $productVariant )->getQty ();
				/* @var $qtyStock Mage_CatalogInventory_Model_Stock_Item */
				
				try {
					$request = array (
							'productId' => $variantId ['via_productId'] . 'L',
							'productVariationId' => $variantId ['via_variationId'] . 'L',
							'stockAmount' => intval ( $qtyStock > 0 ? $qtyStock : 0 ),
							'price' => $mappedAttributes ['Price'] . 'm' 
					);
					
					if ($mappedAttributes ['DiscountOffer']) {
						$request ['DiscountOffer'] = $mappedAttributes ['DiscountOffer'] . 'm';
					}
					
					$this->getClient ()->call ( "ReviseInventoryStatus", $request );
					$result->variation ++;
				} catch ( Exception $e ) {
					$this->log->logException ( $e );
					$result->error ++;
				}
			}
		}
	}
	
	/**
	 * Get Magento categories.
	 *
	 * @return array
	 */
	protected function getMagentoCategories() {
		if ($this->_magentoCategories == null) {
			$rootCategory = $this->_viaDataHelper->getVIARootCategory ();
			if ($rootCategory == null || ! $rootCategory->getIsActive ()) {
				$this->_magentoCategories = array ();
			} else {
				$inactive = Mage::getModel ( 'catalog/category' )->getCollection ();
				$inactive->setStoreId ( $this->_viaDataHelper->getVIAStoreId () );
				$inactive->addAttributeToFilter ( 'is_active', 0 );
				$inactiveIds = $inactive->getAllIds ();
				
				$categoryModel = Mage::getResourceModel ( 'viaebay/category_collection' );
				
				$categoryModel->setDisableFlat ( true );
				$categoryModel->setStoreId ( $this->_viaDataHelper->getVIAStoreId () );
				$categoryModel->addPathsFilter ( $rootCategory->getPath () . '/' );
				$categoryModel->addIsActiveFilter ();
				$categoryModel->addAttributeToSelect ( array (
						'name',
						'viaebay_id' 
				) );
				foreach ( $inactiveIds as $inactiveId ) {
					$categoryModel->addAttributeToFilter ( 'path', array (
							'nlike' => '%/' . $inactiveId . '/%' 
					) );
				}
				
				$mCats = $categoryModel->getItems ();
				
				array_unshift ( $mCats, $rootCategory );
				
				$this->_magentoCategories = $mCats;
			}
		}
		return $this->_magentoCategories;
	}
	
	/**
	 * Get VIA catalogs.
	 *
	 * @return array
	 */
	protected function getViaCatalogs() {
		if ($this->_viaCatalogs == null) {
			$viaCatalogs = $this->getClient ()->loadCatalogs ( null, 'ParentCatalogs' );
			if (! is_array ( $viaCatalogs )) {
				throw new Exception ( 'Could not load Catalogs' );
			}
			
			$this->_viaCatalogs = $viaCatalogs;
		}
		return $this->_viaCatalogs;
	}
	
	/**
	 * Add new view catalog.
	 *
	 * @param array $viaCatalog        	
	 */
	protected function addViaCatalog(array $viaCatalog) {
		// Be sure they are loaded
		$this->getViaCatalogs ();
		
		// Append catalog
		$this->_viaCatalogs [] = $viaCatalog;
	}
	
	/**
	 * Get VIA products.
	 *
	 * @param string $filter        	
	 * @throws Exception
	 * @return mixed
	 */
	protected function getViaProducts($filter = null) {
		$parts = array (
				'CarParts',
				'Catalogs',
				'ChildProducts',
				'DiscountOffers',
				'OptionalProductAttributes',
				'ProductImages',
				'ProductSpecifics',
				'ProductVariations/ProductVariationSpecifics',
				'ProductVariations/ProductVariationPictures' 
		);
		
		$viaProducts = $this->getClient ()->loadProducts ( $filter, implode ( ',', $parts ) );
		if (! is_array ( $viaProducts ) || count ( $viaProducts ) <= 0) {
			$this->log->log ( 'Could not load Products using filter ' . $filter );
			return null;
		}
		return $viaProducts;
	}
	
	/**
	 * Get VIA product by its 'ForeignId'.
	 *
	 * @param string $foreignId        	
	 * @throws Exception
	 */
	protected function getViaProduct($foreignId) {
		if (strlen ( $foreignId ) <= 0) {
			return null;
		}
		
		$count = 0;
		$maxTries = 3;
		
		while ( true ) {
			try {
				$result = $this->getViaProducts ( "ForeignId eq '" . $foreignId . "'" );
				return $result [0];
			} catch ( Exception $e ) {
				$this->log->log ( "Could not load product. Attempt was #" . $count );
				$this->log->logException ( $e );
				if (++ $count >= $maxTries) {
					throw $e;
				}
			}
		}
	}
	
	/**
	 * Get catalog by it's 'Id' from collection.
	 *
	 * @param array $catalogs        	
	 * @param unknown $id        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getCatalog(array $catalogs, $id) {
		return $this->getArrayValueByKeyAndValue ( $catalogs, 'Id', $id );
	}
	
	/**
	 * Get product by it's 'Id' from collection.
	 *
	 * @param array $products        	
	 * @param unknown $id        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getProduct(array $products, $id) {
		return $this->getArrayValueByKeyAndValue ( $products, 'Id', $id );
	}
	
	/**
	 * Get child product by it's id from collection.
	 *
	 * @param array $products        	
	 * @param unknown $id        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getChildProduct(array $products, $id) {
		return $this->getArrayValueByKeyAndValue ( $products, 'ChildId', $id );
	}
	
	/**
	 * Get product image by 'ImageUrl' and 'Type' from collection.
	 *
	 * @param array $images        	
	 * @param unknown $imageUrl        	
	 * @param string $type        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getProductImage(array $images, $imageUrl, $type = null) {
		return $this->getArrayValueByKeyAndValue ( $images, 'ImageUrl', $imageUrl, $type != null ? array (
				'Type' => $type 
		) : array () );
	}
	
	/**
	 * Get product image by 'ImageUrl' and 'Type' from collection.
	 *
	 * @param array $images        	
	 * @param unknown $imageUrl        	
	 * @param string $type        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getProductPicture(array $images, $imageUrl, $position = null) {
		return $this->getArrayValueByKeyAndValue ( $images, 'ImageUrl', $imageUrl, $position != null ? array (
				'ImagePosition' => $position 
		) : array () );
	}
	
	/**
	 * Get product image by 'ImageUrl' and 'Type' from collection.
	 *
	 * @param array $images        	
	 * @param unknown $imageUrl        	
	 * @param string $type        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getProductPictureByImagePosition(array $images, $position) {
		return $this->getArrayValueByKeyAndValue ( $images, 'ImagePosition', $position );
	}
	
	/**
	 * Get entity by 'Name' from collection.
	 *
	 * @param array $arr        	
	 * @param unknown $name        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getByName(array $arr, $name) {
		return $this->getArrayValueByKeyAndValue ( $arr, 'Name', $name );
	}
	
	/**
	 * Get entity by 'Sku' from collection.
	 *
	 * @param array $arr        	
	 * @param unknown $sku        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getBySku(array $arr, $sku) {
		return $this->getArrayValueByKeyAndValue ( $arr, 'Sku', $sku );
	}
	
	/**
	 * Get entity by 'ForeignId' from collection.
	 *
	 * @param array $arr        	
	 * @param unknown $id        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getByForeignId(array $arr, $id) {
		return $this->getArrayValueByKeyAndValue ( $arr, 'ForeignId', $id );
	}
	
	/**
	 * Get entity by 'ProductId' from collection.
	 *
	 * @param array $arr        	
	 * @param unknown $id        	
	 * @param string $variationProductId        	
	 * @return Ambigous <NULL, unknown>
	 */
	protected function getByProductId(array $arr, $id, $variationProductId = null) {
		return $this->getArrayValueByKeyAndValue ( $arr, 'ProductId', $id, $variationProductId != null ? array (
				'ProductVariationId' => $variationProductId 
		) : array () );
	}
	
	/**
	 * Search array for key and value.
	 *
	 * @param array $array        	
	 * @param unknown $key        	
	 * @param unknown $value        	
	 * @param array $fields        	
	 * @return NULL|unknown
	 */
	protected function getArrayValueByKeyAndValue(array $array, $key, $value, array $fields = array()) {
		if ($array == null || ! is_array ( $array )) {
			return null;
		}
		
		if (empty ( $key )) {
			return null;
		}
		
		/*
		 * if (empty ( $value )) {
		 * return null;
		 * }
		 */
		
		$fields [$key] = $value;
		
		foreach ( $array as $item ) {
			foreach ( $fields as $k => $v ) {
				/*
				 * if ($k == 'ImagePosition') {
				 * var_dump ( $k, $v, $item );
				 * }
				 */
				
				if (! isset ( $item [$k] ) || $item [$k] != $v) {
					continue 2;
				}
			}
			return $item;
		}
		return null;
	}
	
	/**
	 * Delete given $viaEntites if they are not listed in $seenEntityIds
	 *
	 * @param array $viaEntites
	 *        	all viaEntity
	 * @param array $seenEntityIds
	 *        	enties already seen
	 */
	protected function deleteEntityIfIdHasNotBeenSeen(array $viaEnties, array $seenEntityIds) {
		$deleted = 0;
		foreach ( $viaEnties as $entity ) {
			try {
				if (! isset ( $entity ['Id'] )) {
					$this->log->logToFile ( "Entity without Id. Probably new.", Zend_Log::DEBUG );
					continue;
				} else if (isset ( $seenEntityIds [$entity ['Id']] )) {
					$this->log->logToFile ( "Don't delete seen entity " . $entity ['Id'], Zend_Log::DEBUG );
					continue;
				}
				
				$metadata = $entity ['__metadata'];
				$type = isset ( $metadata ['type'] ) ? $metadata ['type'] : 'undefined';
				$uri = isset ( $metadata ['uri'] ) ? $metadata ['uri'] : 'undefined';
				$this->log->log ( "Deleting entity of type '" . $type . "' at '" . $uri . "'", Zend_Log::INFO );
				$this->getClient ()->deleteObject ( $entity );
				$deleted ++;
			} catch ( Exception $ex ) {
				$this->log->logException ( $ex );
			}
		}
		return $deleted;
	}
}
