<?php
class VIA_Ebay_Model_Service_Order extends VIA_Ebay_Model_Service_Abstract {
	const USE_GUEST_ACCOUNT_CONFIG = 'viaebay/order/use_guest_account';
	const CUSTOMER_GROUP_CONFIG = 'viaebay/order/customer_group';
	const ORDER_STATUS_CONFIG = 'viaebay/order/order_status';
	const ORDER_IMPORT_FAILED = '_ORDER_IMPORT_FAILED_';
	
	/**
	 * Default constructor.
	 */
	function __construct() {
		parent::__construct ( "viaebay_orders_import" );
	}
	
	/**
	 * Import order.
	 *
	 * @param array|null $ids
	 * @return array
	 */
	public function import(array $ids = null) {
		$result = new Varien_Object ( array (
				'added' => 0,
				'paid' => 0,
				'error' => 0 
		) );
		
		try {
			$this->lock ();
			
			$this->log->log ( 'Starting VIAEbay Order Import ' . Mage::helper ( 'viaebay/data' )->getExtensionVersion (), Zend_Log::INFO );
			
			$expand = 'Address,Buyer,SalesOrderItems/Product,SalesOrderItems/ProductVariation';
			$force = false;
			$updateForeignOrderId = true;
			
			$filter = null;
			if (count ( $ids )) {
				$clauses = array ();
				foreach ( $ids as $id ) {
					$clauses [] = "(Id eq " . $id . ")";
				}
				
				$filter = join ( ' or ', $clauses );
				$force = true;
			} else {
				$filter = "(ForeignOrderId eq null) or (ForeignOrderId eq '')";
			}
			
			/* @var $cookieModel Mage_Core_Model_Cookie */
			$cookieModel = Mage::getModel ( 'core/cookie' );
			
			$cookieFilter = $cookieModel->get ( 'viaebay_order_filter' );
			if ($cookieFilter == "null") {
				$filter = null;
			} else if (strlen ( $cookieFilter ) > 0) {
				$filter = $cookieFilter;
			}
			
			$cookieForce = $cookieModel->get ( 'viaebay_order_force' );
			if ($cookieForce == "true") {
				$force = true;
			}
			
			// $filter = null;
			// $force = true;
			// $updateForeignOrderId = false;
			
			foreach ( $this->getClient ()->loadOrders ( $filter, $expand ) as $viaOrder ) {
				$customOrder = $this->getOrCreateVIAOrder ( $viaOrder );
				
				try {
					if (! isset ( $viaOrder ['Id'] )) {
						throw new Exception ( "Orders without 'Id' cannot be processed " . Zend_Json::encode ( $viaOrder ) );
					}
					
					$order = null;
					
					if (empty ( $viaOrder ['ForeignOrderId'] ) || $force) {
						$this->log->log ( 'Import order', Zend_Log::INFO );
						$order = $this->importNewOrder ( $viaOrder, $customOrder, $updateForeignOrderId );
						if ($order != null) {
							$result->added ++;
						}
					} else {
						$this->log->log ( 'Skip known order m' . $viaOrder ['ForeignOrderId'], Zend_Log::INFO );
					}
					
					if ($viaOrder ['MonetaryPaymentStatus'] == 1) {
						$invoice = $this->createInvoice ( $viaOrder, $order );
						if ($invoice != null) {
							$result->paid ++;
						}
					}
				} catch ( VIA_Ebay_Model_Exception_Products $ex ) {
					$result->error ++;
					$this->log->logException ( $ex );
					/* @var $inbox Mage_AdminNotification_Model_Inbox */
					$inbox = Mage::getModel ( 'adminnotification/inbox' );
					$inbox->setDateAdded ( Mage::getSingleton ( 'core/date' )->gmtDate () );
					$inbox->setSeverity ( Mage_AdminNotification_Model_Inbox::SEVERITY_MAJOR );
					$inbox->setTitle ( Mage::helper ( 'viaebay/data' )->__ ( 'Could not import order' ) );
					$inbox->setDescription ( $customOrder->getViaebayOrderId () . ' ' . $customOrder->getBuyerName () . ' ' . $customOrder->getCheckoutCompleteDate () . ' ' . $ex->__toString () );
					$inbox->setUrl ( $ex->getProductUrl () );
					$inbox->save ();
					if ($updateForeignOrderId) {
						$this->getClient ()->updateObjectField ( $viaOrder, 'ForeignOrderId', self::ORDER_IMPORT_FAILED );
					}
				} catch ( VIA_Ebay_Model_Exception_Product $ex ) {
					$result->error ++;
					$this->log->logException ( $ex );
					/* @var $inbox Mage_AdminNotification_Model_Inbox */
					$inbox = Mage::getModel ( 'adminnotification/inbox' );
					$inbox->setDateAdded ( Mage::getSingleton ( 'core/date' )->gmtDate () );
					$inbox->setSeverity ( Mage_AdminNotification_Model_Inbox::SEVERITY_MAJOR );
					$inbox->setTitle ( Mage::helper ( 'viaebay/data' )->__ ( 'Could not import order' ) );
					$inbox->setDescription ( $customOrder->getViaebayOrderId () . ' ' . $customOrder->getBuyerName () . ' ' . $customOrder->getCheckoutCompleteDate () . ' ' . $ex->getMessage () );
					$inbox->setUrl ( $ex->getProductUrl () );
					$inbox->save ();
					if ($updateForeignOrderId) {
						$this->getClient ()->updateObjectField ( $viaOrder, 'ForeignOrderId', self::ORDER_IMPORT_FAILED );
					}
				} catch ( Exception $ex ) {
					$result->error ++;
					$this->log->logException ( $ex );
					/* @var $inbox Mage_AdminNotification_Model_Inbox */
					$inbox = Mage::getModel ( 'adminnotification/inbox' );
					$inbox->setDateAdded ( Mage::getSingleton ( 'core/date' )->gmtDate () );
					$inbox->setSeverity ( Mage_AdminNotification_Model_Inbox::SEVERITY_MAJOR );
					$inbox->setTitle ( Mage::helper ( 'viaebay/data' )->__ ( 'Could not import order' ) );
					$inbox->setDescription ( $customOrder->getViaebayOrderId () . ' ' . $customOrder->getBuyerName () . ' ' . $customOrder->getCheckoutCompleteDate () . ' ' . $ex->getMessage () );
					$inbox->save ();
					if ($updateForeignOrderId) {
						$this->getClient ()->updateObjectField ( $viaOrder, 'ForeignOrderId', self::ORDER_IMPORT_FAILED );
					}
				}
			}
		} catch ( Exception $e ) {
			$this->log->logException ( $e );
			$result->error ++;
		}
		
		$this->log->log ( 'Completed VIAEbay Order Import', Zend_Log::INFO );
		
		$this->unlock ();
		
		return $result;
	}
	
	/**
	 * Import new order.
	 *
	 * @param array $viaOrder
	 * @param VIA_Ebay_Model_Order $customOrder
	 * @param bool $updateForeignOrderId
	 * @return Mage_Sales_Model_Order
	 * @throws Exception
	 */
	public function importNewOrder(array $viaOrder, VIA_Ebay_Model_Order $customOrder, $updateForeignOrderId = true) {
		$viaOrderId = $viaOrder ['Id'];
		
		/* @var $helper VIA_Ebay_Model_Helper_Data */
		$helper = Mage::helper ( 'viaebay/data' );
		
		/* @var $store Mage_Core_Model_Store */
		$store = $helper->getVIAStore ();
		
		/* @var $taxCalculation Mage_Tax_Model_Calculation */
		$taxCalculation = Mage::getModel ( 'tax/calculation' );
		
		/* @var $taxConfig Mage_Tax_Model_Config */
		$taxConfig = Mage::getSingleton ( 'tax/config' );
		
		$priceIncludesTax = $taxConfig->priceIncludesTax ( $store );
		$shippingIncludesTax = $taxConfig->shippingPriceIncludesTax ( $store );
		
		$buyerName = isset ( $viaOrder ['Buyer'] ['BuyerName'] ) ? $viaOrder ['Buyer'] ['BuyerName'] : '';
		$platformOrderId = $viaOrder ['PlatformOrderId'];
		
		/* @var $checkoutSession Mage_Checkout_Model_Session */
		$checkoutSession = Mage::getSingleton ( 'checkout/session' );
		/* @var $customerSession Mage_Customer_Model_Session */
		$customerSession = Mage::getSingleton ( 'customer/session' );
		
		$viaStore = $helper->getVIAStore ();
		if ($viaStore == null) {
			throw new Exception ( "No VIA-Connect store defined" );
		}
		
		/* @var $orderObj Mage_Sales_Model_Order */
		$orderObj = Mage::getModel ( 'sales/order' )->load ( $viaOrder ['Id'], 'viaebay_id' );
		
		// do not import order already existing orders
		if ($orderObj->getId ()) {
			throw new Exception ( "Order already existing. " . $viaOrder ['Id'] );
		}
		
		$customerSession->clear ();
		$checkoutSession->clear ();
		
		$quote = $checkoutSession->getQuote ();
		$quote->setStore ( $viaStore );
		
		$viaAddress = $viaOrder ['Address'];
		if (isset ( $viaOrder ['ShippingAddress'] ['Id'] )) {
			$viaShippingAddress = $viaOrder ['ShippingAddress'];
		} else {
			$viaShippingAddress = $viaAddress;
		}
		
		$customer = null;
		if (Mage::getStoreConfigFlag ( self::USE_GUEST_ACCOUNT_CONFIG )) {
			$quote->setCustomerId ( null );
			$quote->setCustomerIsGuest ( true );
		} else {
			$customer = $this->getOrCreateCustomer ( $viaOrder );
			$customer->save ();
			$quote->setCustomer ( $customer );
		}
		
		// Set email for guest order. Don't use $customer because guests don't have their name set:
		$quote->setCustomerFirstname ( $viaAddress ['Name'] );
		$quote->setCustomerLastname ( $viaAddress ['Surname'] );
		$quote->setCustomerEmail ( $viaAddress ['Email'] );
		$quote->setCustomerNote ( $viaOrder ['BuyerCheckoutMessage'] );
		
		$convertedBillingAddress = $this->convertAddress ( $viaAddress );
		$quote->getBillingAddress ()->addData ( $convertedBillingAddress );
		$convertedShippingAddress = $this->convertAddress ( $viaShippingAddress );
		$shippingAddress = $quote->getShippingAddress ()->addData ( $convertedShippingAddress );
		
		$rate = $this->calculateRate ( $viaOrder, $quote, $taxCalculation, $shippingIncludesTax );
		
		$shippingAddress->setCollectShippingRates ( false );
		$shippingAddress->addShippingRate ( $rate );
		$shippingAddress->setShippingMethod ( $rate->getCode () );
		
		$shippingAddress->setBaseShippingAmount ( $viaOrder ['ShippingServiceCost'] );
		$shippingAddress->setShippingAmount ( $viaOrder ['ShippingServiceCost'] );
		
		$paymentMethod = $this->convertPaymentMethod ( $viaOrder );
		$viaebayPaymentMethod = $viaOrder ['PaymentOption'];
		
		$shippingAddress->setPaymentMethod ( $paymentMethod );
		
		$this->saveCustomerAddress ( $customer, $convertedShippingAddress, false, false );
		$this->saveCustomerAddress ( $customer, $convertedBillingAddress, false, false );
		
		$this->importProducts ( $viaOrder, $quote, $taxCalculation, $priceIncludesTax );
		
		$this->log->log ( 'Items added', Zend_Log::INFO );
		
		$paymentData = array (
				'method' => $paymentMethod,
				'viaebay_remote_method' => $viaebayPaymentMethod,
				'viaebay_buyer_name' => $buyerName,
				'viaebay_buyer_package_enclosure' => $viaOrder ['BuyerPackageEnclosure'],
				'viaebay_payment_transaction_id' => $viaOrder ['PaymentTransactionId'] 
		);
		
		$payment = $quote->getPayment ();
		
		if (! $payment->getQuote ()) {
			// This should never happen but there are modules causing this.
			$payment->setQuote ( $quote );
		}
		
		$payment->importData ( $paymentData );
		
		// Required for Firegento_MageSetup
		/* @var $checkoutSession Mage_Checkout_Model_Session */
		$checkoutSession = Mage::getSingleton ( 'checkout/session' );
		$checkoutSession->replaceQuote ( $quote );
		
		$quote->setTotalsCollectedFlag ( false )->collectTotals ();
		$quote->setGrandTotal ( $viaOrder ['TotalPrice'] );
		$quote->save ();
		
		// Required for Firegento_MageSetup
		$checkoutSession->clear ();
		
		$this->log->log ( 'Quote complete', Zend_Log::INFO );
		
		/* @var $service Mage_Sales_Model_Service_Quote */
		$service = Mage::getModel ( 'sales/service_quote', $quote );
		$service->submitAll ();
		
		$this->log->log ( 'Quote submitted', Zend_Log::INFO );
		
		/* @var $order Mage_Sales_Model_Order */
		$order = $service->getOrder ();
		
		$order->setStatus ( Mage::getStoreConfig ( self::ORDER_STATUS_CONFIG ) );
		if (isset ( $viaOrder ['Buyer'] ['BuyerName'] )) {
			$order->addStatusHistoryComment ( Mage::helper ( 'viaebay/data' )->__ ( 'Buyer: %s', $viaOrder ['Buyer'] ['BuyerName'] ) );
		}
		
		if (strlen ( $platformOrderId )) {
			$order->addStatusHistoryComment ( $helper->__ ( 'Platform Order # %s', $platformOrderId ) );
		}
		
		if (strlen ( $viaOrder ['BuyerCheckoutMessage'] )) {
			$order->addStatusHistoryComment ( $viaOrder ['BuyerCheckoutMessage'] );
		}
		
		$order->getPayment ()->setLastTransId ( $viaOrder ['PaymentTransactionId'] );
		$order->setData ( 'viaebay_id', $viaOrderId );
		$order->save ();
		
		// Store order id to our own table
		$customOrder->setMagentoOrderId ( $order->getId () );
		$customOrder->save ();
		
		$this->log->log ( 'Order created m' . $order->getIncrementId (), Zend_Log::INFO );
		
		if ($updateForeignOrderId) {
			$this->getClient ()->updateObjectField ( $viaOrder, 'ForeignOrderId', $order->getId () );
		}
		
		$this->log->log ( 'Update stock', Zend_Log::INFO );
		
		foreach ( $order->getItemsCollection () as $item ) {
			/* @var $item Mage_Sales_Model_Order_Item */
			Mage::getSingleton ( 'viaebay/service_catalog' )->updateStock ( $item->getProduct () );
		}
		
		try {
			$order->sendNewOrderEmail ();
		} catch ( Exception $e ) {
			$this->log->logException ( $e, false );
		}
		
		Mage::dispatchEvent ( 'viaebay_order_new', array (
				'order' => $order 
		) );
		
		return $order;
	}
	
	/**
	 * Import products to quote.
	 *
	 * @param array $viaOrder
	 * @param Mage_Sales_Model_Quote $quote
	 * @throws VIA_Ebay_Model_Exception_Product
	 * @throws VIA_Ebay_Model_Exception_NoProduct
	 * @throws VIA_Ebay_Model_Exception_Products
	 */
	protected function importProducts(array $viaOrder, Mage_Sales_Model_Quote $quote, Mage_Tax_Model_Calculation $taxCalculation, $priceIncludesTax) {
		/* @var $helper VIA_Ebay_Model_Helper_Data */
		$helper = Mage::helper ( 'viaebay/data' );
		
		/* @var $store Mage_Core_Model_Store */
		$store = $helper->getVIAStore ();
		
		$productItemList = array ();
		$productExceptions = new VIA_Ebay_Model_Exception_Products ();
		
		foreach ( $viaOrder ['SalesOrderItems'] as $viaSalesOrderItem ) {
			$productId = '';
			$productName = '';
			
			try {
				/* @var $product Mage_Catalog_Model_Product */
				if (isset ( $viaSalesOrderItem ['ProductVariation'] ) && isset ( $viaSalesOrderItem ['ProductVariation'] ['ForeignId'] ) && strlen ( $viaSalesOrderItem ['ProductVariation'] ['ForeignId'] ) > 0) {
					$productId = $viaSalesOrderItem ['ProductVariation'] ['ForeignId'];
					$product = Mage::getModel ( 'viaebay/product' )->load ( $productId );
				} elseif (isset ( $viaSalesOrderItem ['ProductVariation'] ) && isset ( $viaSalesOrderItem ['ProductVariation'] ['Sku'] ) && strlen ( $viaSalesOrderItem ['ProductVariation'] ['Sku'] ) > 0) {
					$viaSku = $viaSalesOrderItem ['ProductVariation'] ['Sku'];
					$product = Mage::getModel ( 'viaebay/product' )->loadByAttribute ( 'sku', $viaSku );
				} elseif (isset ( $viaSalesOrderItem ['Product'] ['ForeignId'] )) {
					$productId = $viaSalesOrderItem ['Product'] ['ForeignId'];
					$product = Mage::getModel ( 'viaebay/product' )->load ( $productId );
				} elseif (isset ( $viaSalesOrderItem ['ProductVariationForeignId'] )) {
					$productId = $viaSalesOrderItem ['ProductVariationForeignId'];
					$product = Mage::getModel ( 'viaebay/product' )->load ( $productId );
				} else {
					$productId = $viaSalesOrderItem ['ForeignId'];
					$product = Mage::getModel ( 'viaebay/product' )->load ( $productId );
				}
				
				if ($product == null || ! $product->getId ()) {
					throw new VIA_Ebay_Model_Exception_Product ( $viaSalesOrderItem ['Product'] ['ForeignId'], $helper->__ ( "Could not find product %s v%s for order v%s", $viaSalesOrderItem ['Name'], $viaSalesOrderItem ['Id'], $viaOrder ['Id'] ) );
				}
				
				$productName = $product->getName ();
				
				if ($product->getTypeId () == Mage_Catalog_Model_Product_Type::TYPE_CONFIGURABLE) {
					// Ignored
					continue;
				}
				
				if ($product->getStatus () == Mage_Catalog_Model_Product_Status::STATUS_DISABLED) {
					throw new VIA_Ebay_Model_Exception_Product ( $viaSalesOrderItem ['Product'] ['ForeignId'], $helper->__ ( "Product %s v%s is disabled for order v%s", $viaSalesOrderItem ['Name'], $viaSalesOrderItem ['Id'], $viaOrder ['Id'] ) );
				}
				
				$this->log->log ( 'Add item ' . $viaSalesOrderItem ['Name'], Zend_Log::INFO );
				
				$item = null;
				/* @var $item Mage_Sales_Model_Quote_Item */
				
				if (array_key_exists ( $product->getId (), $productItemList )) {
					// Load $item from cache
					$item = $productItemList [$product->getId ()]->addQty ( $viaSalesOrderItem ['Amount'] );
				} else if (! ($product->getStockItem () instanceof Mage_CatalogInventory_Model_Stock_Item)) {
					// Fix a strange bug when $product->getStockStatus returns a VarienObject
					$stock = Mage::getModel ( 'cataloginventory/stock_item' )->loadByProduct ( $product );
					$product->setStockItem ( $stock );
				}
				
				$stockItem = $product->getStockItem ();
				// Fake stock status
				if (! $stockItem->getIsInStock ()) {
					throw new VIA_Ebay_Model_Exception_Product ( $viaSalesOrderItem ['Product'] ['ForeignId'], $helper->__ ( "Product not in stock %s v%s for order v%s", $viaSalesOrderItem ['Name'], $viaSalesOrderItem ['Id'], $viaOrder ['Id'] ) );
				}
				
				if ($item == null) {
					// $item is not yet loaded
					$item = $quote->addProduct ( $product, $viaSalesOrderItem ['Amount'] );
					
					if (is_string ( $item )) {
						// Check that $item is not a string. This would be an error message.
						throw new VIA_Ebay_Model_Exception_Product ( $product->getId (), $helper->__ ( "Could not create item for product m%s : %s for order v%s", $product->getId (), $item, $viaOrder ['Id'] ) );
					} else if (! is_object ( $item )) {
						// Check that $item is finally loaded
						throw new VIA_Ebay_Model_Exception_Product ( $product->getId (), $helper->__ ( "Could not create item for product m%s for order v%s", $product->getId (), $viaOrder ['Id'] ) );
					}
					
					$taxRequest = $taxCalculation->getRateRequest ( $quote->getShippingAddress (), $quote->getBillingAddress (), $quote->getCustomer (), $store );
					$taxRequest->setProductClassId ( $item->getProduct ()->getTaxClassId () );
					$percent = $taxCalculation->getRate ( $taxRequest );
					if ($percent <= 0) {
						$percent = 19;
					}
					
					if ($priceIncludesTax) {
						$customPrice = $viaSalesOrderItem ['Price'];
					} else {
						$customPrice = $helper->priceWithoutTax ( $viaSalesOrderItem ['Price'], $percent );
					}
					
					$item->setStoreId ( $store->getId () );
					$item->setCustomPrice ( $customPrice );
					$item->setOriginalCustomPrice ( $customPrice );
					$item->getProduct ()->setIsSuperMode ( true );
				}
				
				// Add $item to cache for next rounds
				$productItemList [$product->getId ()] = $item;
			} catch ( Mage_Core_Exception $ex ) {
				$productExceptions->addException ( new VIA_Ebay_Model_Exception_Product ( $productId, $helper->__ ( "Could not import product m%s %s caused by %s", $productId, $productName, $ex->getMessage () ), 0, $ex ) );
			} catch ( VIA_Ebay_Model_Exception_Product $ex ) {
				$productExceptions->addException ( $ex );
			}
		}
		
		if ($productExceptions->hasProductExceptions ()) {
			throw $productExceptions;
		}
		
		if (! $quote->hasItems ()) {
			$this->log->logException ( $productExceptions );
			throw new VIA_Ebay_Model_Exception_NoProduct ( $helper->__ ( "Cannot import order v%s without products", $viaOrder ['Id'] ), $productExceptions->hasProductExceptions () ? $productExceptions : null );
		}
	}
	
	/**
	 *
	 * @param array $viaOrder
	 * @param Mage_Sales_Model_Order $order
	 * @return Mage_Sales_Model_Order_Invoice
	 */
	public function createInvoice(array $viaOrder, Mage_Sales_Model_Order $order = null) {
		/* @var $helper VIA_Ebay_Model_Helper_Data */
		$helper = Mage::helper ( 'viaebay/data' );
		
		if ($order == null) {
			$order = Mage::getModel ( 'sales/order' )->load ( $viaOrder ['ForeignOrderId'] );
		}
		
		if (! $order->getId ()) {
			throw new Exception ( $helper->__ ( "Cannot create invoice without Magento order for order %s", $viaOrder ['Id'] ) );
		}
		
		switch ($order->getPayment ()->getMethod ()) {
			case 'viaebay_paypal' :
			case 'viaebay_cashondelivery' :
				if (strlen ( $viaOrder ['PaymentTransactionId'] ) > 0) {
					$captureCase = Mage_Sales_Model_Order_Invoice::CAPTURE_ONLINE;
				} else {
					return null; // Skip unpayed
				}
				break;
			default :
				$captureCase = Mage_Sales_Model_Order_Invoice::NOT_CAPTURE;
				
				// We don't create unpaid invoices
				return null; // break; if we want to create not captured invoices
		}
		
		if (! $order->canInvoice ()) {
			return null;
		}
		
		$this->log->log ( 'Create invoice for m' . $order->getId (), Zend_Log::INFO );
		
		$invoice = $order->prepareInvoice ();
		/* @var $invoice Mage_Sales_Model_Order_Invoice */
		
		if (! $invoice->getTotalQty ()) {
			throw new Exception ( $helper->__ ( "Cannot create an invoice without products for order m%s", $order->getId () ) );
		}
		
		$invoice->setRequestedCaptureCase ( $captureCase );
		$invoice->register ();
		
		$invoice->setIsInProcess ( true );
		
		Mage::getModel ( 'core/resource_transaction' )->addObject ( $invoice )->addObject ( $invoice->getOrder () )->save ();
		
		$this->log->log ( 'Created invoice m' . $invoice->getId () . ' for order m' . $order->getId (), Zend_Log::INFO );
		
		try {
			$invoice->sendEmail ();
		} catch ( Exception $e ) {
			$this->log->logException ( $e, false );
		}
		
		Mage::dispatchEvent ( 'viaebay_order_invoice_new', array (
				'order' => $order,
				'invoice' => $invoice 
		) );
		
		return $invoice;
	}
	
	/**
	 *
	 * @param Mage_Sales_Model_Order_Shipment_Track $track
	 */
	public function addTrackingNumber(Mage_Sales_Model_Order_Shipment_Track $track) {
		$shipment = $track->getShipment ();
		/* @var $order Mage_Sales_Model_Order_Shipment */
		$order = $shipment->getOrder ();
		/* @var $order Mage_Sales_Model_Order */
		
		$carrier = $shipment->getOrder ()->getShippingCarrier ();
		if ($carrier == null || ! $carrier instanceof VIA_Ebay_Model_Carrier) {
			return;
		}
		
		$carrierUsed = $carrier->getViaebayCarrier ( $order->getShippingMethod () );
		if ($carrierUsed == null || strlen ( $carrierUsed ) <= 0) {
			return;
		}
		
		$viaOrders = $this->getClient ()->loadOrders ( "ForeignOrderId eq '" . $order->getId () . "'", 'SalesOrderItems' );
		
		$trackingNumber = $track->getNumber ();
		
		foreach ( $shipment->getItemsCollection () as $shipmentItem ) {
			$productId = $shipmentItem->getProductId ();
			if ($productId == null) {
				continue;
			}
			
			foreach ( $viaOrders as $viaOrder ) {
				foreach ( $viaOrder ['SalesOrderItems'] as $salesOrderItem ) {
					if (isset ( $salesOrderItem ['ProductVariationForeignId'] ) && $salesOrderItem ['ProductVariationForeignId'] == $productId || isset ( $salesOrderItem ['ForeignId'] ) && $salesOrderItem ['ForeignId'] == $productId) {
						$trackingNumbers = explode ( ";", $salesOrderItem ['TrackingNumbers'] );
						$trackingNumbers [] = $trackingNumber;
						array_unique ( $trackingNumbers );
						$trackingNumbersString = join ( ";", $trackingNumbers );
						
						if (strlen ( $trackingNumbersString ) > 0 && $trackingNumbersString != $salesOrderItem ['TrackingNumbers']) {
							$delta = array (
									'TrackingNumbers' => $trackingNumbersString,
									'CarrierUsed' => $carrierUsed 
							);
							
							$this->log->log ( 'Set TrackingNumbers for v' . $viaOrder ['Id'] . ' to ' . $trackingNumbers, Zend_Log::INFO );
							try {
								$this->updateObject ( $salesOrderItem, $delta );
							} catch ( Exception $e ) {
								$this->log->log ( 'Failed setting TrackingNumbers for v' . $viaOrder ['Id'] . ' to ' . $trackingNumbers, Zend_Log::ERR );
								$this->log->logException ( $e );
							}
						}
					}
				}
			}
		}
	}
	
	/**
	 *
	 * @param Mage_Sales_Model_Order_Shipment $shipment
	 */
	public function updateShipmentStatus(Mage_Sales_Model_Order_Shipment $shipment) {
		$viaOrders = $this->getClient ()->loadOrders ( "ForeignOrderId eq '" . $shipment->getOrder ()->getId () . "'", 'SalesOrderItems' );
		
		foreach ( $shipment->getItemsCollection () as $shipmentItem ) {
			$productId = $shipmentItem->getProductId ();
			if ($productId == null) {
				continue;
			}
			
			foreach ( $viaOrders as $viaOrder ) {
				// Should be just one matching order
				foreach ( $viaOrder ['SalesOrderItems'] as $salesOrderItem ) {
					if ($salesOrderItem ['ShippingStatus'] != 1 && (isset ( $salesOrderItem ['ProductVariationForeignId'] ) && $salesOrderItem ['ProductVariationForeignId'] == $productId || isset ( $salesOrderItem ['ForeignId'] ) && $salesOrderItem ['ForeignId'] == $productId)) {
						try {
							$this->log->log ( 'Set ShipmentStatus to 1 for v' . $salesOrderItem ['Id'], Zend_Log::INFO );
							$this->getClient ()->updateObjectField ( $salesOrderItem, 'ShippingStatus', 1 );
						} catch ( Exception $e ) {
							$this->log->log ( 'Failed setting ShipmentStatus to 1 for v' . $salesOrderItem ['Id'], Zend_Log::ERR );
							$this->log->logException ( $e );
						}
					}
				}
			}
		}
	}
	
	/**
	 *
	 * @param Mage_Sales_Model_Order $order
	 */
	public function updatePaid(Mage_Sales_Model_Order_Payment $payment, $amount = null) {
		$order = $payment->getOrder ();
		$viaOrders = $this->getClient ()->loadOrders ( "ForeignOrderId eq '" . $order->getId () . "'" );
		
		if ($amount === null) {
			$amount = $payment->getAmountPaid ();
		}
		
		if (is_array ( $viaOrders )) {
			foreach ( $viaOrders as $viaOrder ) {
				if ($viaOrder ['PaymentOption'] != 'Paypal' && abs ( ( float ) $viaOrder ['PaidAmount'] - ( float ) $amount ) > 0.001) {
					$this->log->log ( 'Set PaidAmount for v' . $viaOrder ['Id'] . ' to ' . $amount, Zend_Log::INFO );
					try {
						$this->getClient ()->updateObjectField ( $viaOrder, 'PaidAmount', strval ( $amount ) );
					} catch ( Exception $e ) {
						$this->log->log ( 'Failed setting PaidAmount for v' . $viaOrder ['Id'] . ' to ' . $amount, Zend_Log::ERR );
						$this->log->logException ( $e );
					}
				}
			}
		} else {
			$this->log->log ( 'Could not load via order for m' . $order->getId (), Zend_Log::WARN );
		}
	}
	
	/**
	 * Convert via address to magento address.
	 *
	 * @param array $viaAddress
	 *        	via address to convert.
	 * @return array magento address array
	 */
	protected function convertAddress(array $viaAddress) {
		$countryId = strtoupper ( $viaAddress ['Country'] );
		
		$address = array (
				'company' => $viaAddress ['Company'] == "-" ? "" : $viaAddress ['Company'],
				'firstname' => $viaAddress ['Name'],
				'lastname' => $viaAddress ['Surname'],
				'street' => $viaAddress ['Street'],
				'city' => $viaAddress ['Town'],
				'postcode' => $viaAddress ['PostalCode'],
				'telephone' => $viaAddress ['Phone'],
				'country_id' => $countryId,
				'region_id' => $viaAddress ['State'],
				'email' => $viaAddress ['Email'] 
		);
		
		/**
		 * Zend_Debug::dump($viaAddress);
		 * Zend_Debug::dump($address);
		 * die;
		 */
		
		return $address;
	}
	
	/**
	 * Save customer address if new
	 *
	 * @param Mage_Customer_Model_Customer $customer
	 * @param array $data
	 * @param string $shipping
	 * @param string $billing
	 */
	protected function saveCustomerAddress(Mage_Customer_Model_Customer $customer, array $newAddressData, $shipping = false, $billing = false) {
		if ($customer && $customer->getId ()) {
			/* @var $addressModel Mage_Customer_Model_Address */
			$newAddressModel = Mage::getModel ( 'customer/address' );
			$newAddressModel->setData ( $newAddressData )->setCustomerId ( $customer->getId () )->setIsDefaultBilling ( $billing )->setIsDefaultShipping ( $shipping )->setSaveInAddressBook ( '1' );
			
			$newAddressArray = $newAddressModel->getData ();
			
			$found = false;
			$excludeKeys = array (
					'entity_id',
					'customer_address_id',
					'quote_address_id',
					'region_id',
					'customer_id',
					'address_type',
					'email',
					'is_default_billing',
					'is_default_shipping',
					'save_in_address_book' 
			);
			$addressArrayFiltered = array_diff_key ( $newAddressArray, array_flip ( $excludeKeys ) );
			
			foreach ( $customer->getAddresses () as $address ) {
				/* @var $address Mage_Customer_Model_Address */
				
				$customerAddressArrayFiltered = array_diff_key ( $address->getData (), array_flip ( $excludeKeys ) );
				
				$addressDiff = array_diff_assoc ( $addressArrayFiltered, $customerAddressArrayFiltered );
				
				if (empty ( $addressDiff )) {
					$found = true;
					break;
				}
			}
			
			if (! $found) {
				try {
					$newAddressModel->save ();
				} catch ( Exception $ex ) {
					$this->log->logException ( $ex );
				}
			}
		}
	}
	
	/**
	 * Calculate carrier rate.
	 *
	 * @param array $viaOrder
	 *        	to convert
	 * @return Mage_Sales_Model_Quote_Address_Rate rate calculated
	 */
	protected function calculateRate(array $viaOrder, Mage_Sales_Model_Quote $quote, Mage_Tax_Model_Calculation $taxCalculation, $shippingIncludesTax) {
		/* @var $helper VIA_Ebay_Model_Helper_Data */
		$helper = Mage::helper ( 'viaebay/data' );
		
		/* @var $store Mage_Core_Model_Store */
		$store = $helper->getVIAStore ();
		
		$carrier = Mage::getModel ( 'viaebay/carrier' );
		/* @var $carrier VIA_Ebay_Model_Carrier_Default */
		$methods = VIA_Ebay_Model_Carrier::getAllMethods ();
		$result = Mage::getModel ( 'sales/quote_address_rate' );
		/* @var $result Mage_Sales_Model_Quote_Address_Rate */
		
		$taxRequest = $taxCalculation->getRateRequest ( $quote->getShippingAddress (), $quote->getBillingAddress (), $quote->getCustomer (), $store );
		$taxRequest->setProductClassId ( Mage::getStoreConfig ( 'tax/classes/shipping_tax_class', $store ) );
		$percent = $taxCalculation->getRate ( $taxRequest );
		if ($percent <= 0) {
			$percent = 19;
		}
		
		if ($shippingIncludesTax) {
			$shippingPrice = $viaOrder ['ShippingServiceCost'];
		} else {
			$shippingPrice = $helper->priceWithoutTax ( $viaOrder ['ShippingServiceCost'], $percent );
		}
		
		$result->setCarrierTitle ( 'VIA-Connect' );
		$result->setCode ( 'viaebay_' . $viaOrder ['ShippingService'] );
		$result->setCarrier ( 'viaebay' );
		$result->setMethod ( $viaOrder ['ShippingService'] );
		$result->setMethodTitle ( $methods [$viaOrder ['ShippingService']] );
		$result->setMethodDescription ( $viaOrder ['ShippingService'] );
		$result->setCost ( $shippingPrice );
		$result->setPrice ( $shippingPrice );
		
		return $result;
	}
	
	/**
	 * Get or create customer.
	 *
	 * @param array $viaOrder
	 *        	to check
	 * @return Mage_Customer_Model_Customer
	 */
	protected function getOrCreateCustomer(array $viaOrder) {
		/* @var $helper VIA_Ebay_Model_Helper_Data */
		$helper = Mage::helper ( 'viaebay/data' );
		$viaStore = $helper->getVIAStore ();
		
		$viaAddress = $viaOrder ['Address'];
		$viaBuyer = $viaOrder ['Buyer'];
		
		/* @var $customer Mage_Customer_Model_Customer */
		$customer = Mage::getModel ( 'customer/customer' )->setStore ( $viaStore )->getCollection ()->addAttributeToSelect ( 'viaebay_buyer_id' )->addAttributeToFilter ( 'viaebay_buyer_id', $viaBuyer ['Id'] )->load ()->getFirstItem ();
		
		if (is_object ( $customer ) && $customer->getId () != null) {
			return $customer;
		}
		
		// Reset customer
		$customer = Mage::getModel ( 'customer/customer' );
		
		// Set store and website for loadByEmail.
		$customer->setStore ( $viaStore );
		// Could not find customer by viaebay_buyer_id. As there can only be one customer for a given
		// email try to load one.
		$customer->loadByEmail ( $viaAddress ['Email'] );
		
		if (is_object ( $customer ) && $customer->getId () != null) {
			return $customer;
		}
		
		// New customer
		
		// Set store and website again after loadByEmail reset it.
		$customer->setId ( null );
		$customer->setStore ( $viaStore );
		$customer->setFirstname ( $viaAddress ['Name'] );
		$customer->setLastname ( $viaAddress ['Surname'] );
		$customer->setEmail ( $viaAddress ['Email'] );
		
		$groupId = Mage::getStoreConfig ( self::CUSTOMER_GROUP_CONFIG );
		$customer->setData ( 'group_id', $groupId );
		
		$customer->setData ( 'viaebay_buyer_id', $viaBuyer ['Id'] );
		$customer->setData ( 'viaebay_buyer_name', $viaBuyer ['BuyerName'] );
		
		// recurring customer
		$customer->setPasswordHash ( $customer->generatePassword () );
		
		return $customer;
	}
	
	/**
	 * Convert via payment method to magento payment method
	 *
	 * @param array $viaOrder
	 *        	to convert
	 * @throws Exception when unknown shipping method is found
	 * @return string magento shipping method
	 */
	protected function convertPaymentMethod(array $viaOrder) {
		switch ($viaOrder ['PaymentOption']) {
			case 'CashOnPickup' :
				return 'viaebay_cashonpickup';
			case 'COD' :
				return 'viaebay_cashondelivery';
			case 'MoneyXferAccepted' :
				return 'viaebay_moneyxferaccepted';
			case 'MoneyXferAcceptedInCheckout' :
				return 'viaebay_moneyxferacceptedincheckout';
			case 'PayPal' :
				return 'viaebay_paypal';
			case 'PaymentSeeDescription' :
				return 'viaebay_paymentseedescription';
			case 'Other' :
				return 'viaebay_other';
			default :
				throw new Exception ( Mage::helper ( 'viaebay/data' )->__ ( "Unknown shopping method %s", $viaOrder ['PaymentOption'] ) );
		}
	}
	
	/**
	 *
	 * @param array $viaOrder
	 * @return VIA_Ebay_Model_Order
	 */
	protected function getOrCreateVIAOrder(array $viaOrder) {
		$viaOrderId = $viaOrder ['Id'];
		$buyerName = isset ( $viaOrder ['Buyer'] ['BuyerName'] ) ? $viaOrder ['Buyer'] ['BuyerName'] : '';
		$platformOrderId = $viaOrder ['PlatformOrderId'];
		$date = $this->parseDate ( 'CheckoutCompletionDate', $viaOrder );
		
		/* @var $customOrder VIA_Ebay_Model_Order */
		$customOrder = Mage::getModel ( 'viaebay/order' )->load ( $viaOrderId, 'viaebay_order_id' );
		
		if (! $customOrder->getId ()) {
			$customOrder->setViaebayOrderId ( $viaOrderId );
			$customOrder->setBuyerName ( $buyerName );
			$customOrder->setPlatformOrderId ( $platformOrderId );
			$customOrder->setCheckoutCompleteDate ( $date );
			$customOrder->setMessage ( '' );
			$customOrder->setError ( '' );
			$customOrder->save ();
		}
		
		return $customOrder;
	}
}
